package com.zhengyiyu.feature;

import com.zhengyiyu.handler.DiNucleotideHandler;
import com.zhengyiyu.util.ArrayUtil;

public class DNAStructuralFeature extends SequenceFeature {
	/**
	 * the conversion table for structural features, currently can only be dinucleotide structural features
	 */
	private double[] conversionArray;

	public double[] getConversionArray() {
		return conversionArray;
	}
	
	public void setConversionArray(double[] conversionArray) {
		this.conversionArray = conversionArray;
	}
	
	public DNAStructuralFeature(String propertyName, double[] propertyConversionArray) {
		this(propertyName, propertyConversionArray, "");
	}
	
	public DNAStructuralFeature(String propertyName, double[] propertyConversionArray, String label, double[] cutoffs) {
		this(propertyName, propertyConversionArray, "");
		this.setPatternName(label);
		
		for (double d : cutoffs) {
			this.getDiscretizationCutoff().add(d);
		}
	}
	
	public DNAStructuralFeature(String propertyName, double[] propertyConversionArray, String intro) {
		super();
		this.setFeatureName(propertyName);
		this.conversionArray = propertyConversionArray;
		this.setReference(intro);
	}

	/* (non-Javadoc)
	 * @see com.zhengyiyu.weka.SequenceFeature#calculateFeatureValue(java.lang.String)
	 * currently we can only support dinucleodtide structural features
	 */
	@Override
	public double calculateFeatureValue(String sequence) {
		return ArrayUtil.sumDoubleList(DiNucleotideHandler.calculateDinucleotidePropertyProfile(this.getConversionArray(), sequence)) / (sequence.length() - 1);
	}
}
