package com.zhengyiyu.feature;

/**
 * @author Yiyu
 * The Structural Features is just a double array of the conversion, you will need NucleotideUtil class to calculate the profile of a sequence
 * subfix D means the conversion array for the property is doubtable
 */
public class DNAStructuralFeatureConversionTable {
	// EP3, Ivanov 1994
	public static double[] APHILICITY = {0.97, 0.13, 0.33, 0.58, 1.04, 0.19, 0.52, 0.33, 0.98, 0.73, 0.19, 0.13, 0.73, 0.98, 1.04, 0.97};
	
	// EP3, Gorin 1995
	public static double[] BDNATWIST = {35.5, 33.1, 30.6, 43.2, 37.7, 35.3, 31.3, 30.6, 39.6, 38.4, 35.3, 33.1, 31.6, 39.6, 37.7, 35.5};

	// EP3, Olson 1998
	public static double[] PROTEIN_INDUCED_DEFORMABILITY = {2.9, 2.3, 2.1, 1.6, 9.8, 6.1, 12.1, 2.1, 4.5, 4.0, 6.1, 2.3, 6.3, 4.5, 9.8, 2.8};

	// DIPRODB 34, Delcourt 1991, ... doubtable
	public static double[] DENATURATION_TEMPERATURE_D = {-0.67, -1.28, -1.17, -0.62, -1.19, -1.55, -1.87, -1.17, -1.12, -1.85, -1.55, -1.28, -0.7, -1.12, -1.19, -0.67};

	// DIPRODB 35, free energy, ... doubtable
	public static double[] DISRUPTION_ENERGY_D =  {-1.66, -1.13, -1.35, -1.19, -1.8, -2.75, -3.28, -1.35, -1.41, -2.82, -2.75, -1.13, -0.76, -1.41, -1.8, -1.66};

	// EP3, Breslauer 1986
	public static double[] DISRUPTION_ENERGY_EP3_DUPLEXSTABILITY = {1.9, 1.3, 1.6, 0.9, 1.9, 3.1, 3.6, 1.6, 1.6, 3.1, 3.1, 1.3, 1.5, 1.6, 1.9, 1.9};
	
	// DIPRODB 20, Gorin 1995 not El Hassan 1996
	// confirmed by Lee2007
	public static double[] PROPELLER_TWIST_D = {-17.3, -6.7, -14.3, -16.9, -8.6, -12.8, -11.2, -14.3, -15.1, -11.7, -12.8, -6.7, -11.1, -15.1, -8.6, -17.3};
	
	// EP3, El Hassan 1996
	public static double[] PROPELLER_TWIST_EP3 = {-18.66, -13.10, -14.00, -15.01, -9.45, -8.11, -10.03, -14.00, -13.48, -11.08, -8.11, -13.1, -11.85, -13.48, -9.45, -18.66};
	
	// EP3, Olson 1998, Protein BDNA TWIST, not BDNA TWIST from Gorin 1995, same as Diprodb 37 Twist (DNA-protein complex)
	public static double[] PROTEIN_INDUCED_BDNA_TWIST = {35.1, 31.5, 31.9, 29.3, 37.3, 32.9, 36.1, 31.9, 36.3, 33.6, 32.9, 31.5, 37.8, 36.3, 37.3, 35.1};

	// EP3, Sugimoto 1996, Duplex Stability Free Energy, same as Diprodb 72 free energy
	public static double[] STABILIZATION_ENERGY = {-1.2, -1.5, -1.5, -0.9, -1.7, -2.1, -2.8, -1.5, -1.5, -2.3, -2.1, -1.5, -0.9, -1.5, -1.7, -1.2};
	
	// EP3, Ornstein 1978, same as DIPRODB 33 stacking energy
	public static double[] BASE_STACKING_ENERGY = {-5.37, -10.51, -6.78, -6.57, -6.57, -8.26, -9.69, -6.78, -9.81, -14.59, -8.26, -10.51, -3.82, -9.81, -6.57, -5.37};
	
	// EP3, Sivolob 1995, DNA Bending stiffness, same as DIPRODB 15
	public static double[] PERSISTENCE_LENGTH = {35, 60, 60, 20, 60, 130, 85, 60, 60, 85, 130, 60, 20, 60, 60, 35};
	
	// EP3, Ho1990, Stabilizing energy of Z-DNA, Ho1986 also has the same data
	public static double[] ZDNA_FREEENERGY = {3.9, 4.6, 3.4, 5.9, 1.3, 2.4, 0.7, 3.4, 3.4, 4.0, 2.4, 4.6, 2.5, 3.4, 1.3, 3.9};
	
	// EP3
	public static double[] CPGISLANDS = {0, 0, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 0};
	
	// EP3
	public static double[] DNADENATURATION = {66.51, 108.80, 85.12, 72.29, 64.92, 99.31, 88.84, 85.12, 80.03, 135.83, 99.31, 108.80, 50.11, 80.03, 64.92, 66.51};

	/*
	 * The following are dna di-nucleotides that will be used in Nucleosome prediction 
	 * used in Lee2007
	 * source1: http://srs6.bionet.nsc.ru/srs6bin/cgi-bin/wgetz?-id+6F1Iv1SJDVl+-e+[PROPERTY:'P0000037']
	 * source2: http://wwwmgs.bionet.nsc.ru/mgs/programs/dnaprop/
	 * source3: http://hugheslab.ccbr.utoronto.ca/supplementary-data/tillo/nucleosomes/yeast_struct/dinucleotide_features_newest.xls
	 */
	
	// from Diprodb, confirmed by P0000004
	// confirmed by Lee2007
	public static double[] TIP = {1.76, 2, 0.9, 1.87, -1.64, 0.71, 0.22, 0.9, 1.35, 2.5, 0.71, 2, 6.7, 1.35, -1.64, 1.76}; // number: 5
	
	// from Lee2007 dinucleotide_features_news.xls
	public static double[] TILT = {-0.4, -0.9, -2.6, 0, 0.6, -1.1, 0, -2.6, -0.4, 0, -1.1, -0.9, 0, -0.4, 0.6, -0.4};
	
	// from Diprodb, confirmed by P0000030, and B-DNA Propeller
	// same as the original one : PROPELLER_TWIST_D
	// confirmed by Lee2007
	public static double[] PROPELLERTWIST = PROPELLER_TWIST_D;
//		private static double[] PROPELLERTWIST = {-17.3, -6.7, -14.3, -16.9, -8.6, -12.8, -11.2, -14.3, -15.1, -11.7, -12.8, -6.7, -11.1, -15.1, -8.6, -17.3}; // number: 20
		
	// from Diprodb, confirmed by P0000022, and B-DNA melting temperature
	public static double[] MELTING_TEMPERATURE = {54.5, 97.73, 58.42, 57.02, 54.71, 85.97, 72.55, 58.42, 86.44, 136.12, 85.97, 97.73, 36.73, 86.44, 54.71, 54.5}; // number: 16
	
	// from Diprodb, confirmed by P0000036, and B-DNA Enthalpy change
	// confirmed by Lee2007
	public static double[] ENTHALPY = {-8, -9.4, -6.6, -5.6, -8.2, -10.9, -11.8, -6.6, -8.8, -10.5, -10.9, -9.4, -6.6, -8.8, -8.2, -8}; // number: 22
	
	// from Diprodb, confirmed by P0000037, and B-DNA Entropy change
	// confirmed by Lee2007
	public static double[] ENTROPY = {-21.9, -25.5, -16.4, -15.2, -21, -28.4, -29, -16.4, -23.5, -26.4, -28.4, -25.5, -18.4, -23.5, -21, -21.9}; // number: 23
	
	// from Diprodb, confirmed by P0000035, and B-DNA clash strength
	// confirmed by Lee2007
	public static double[] CLASH_STRENGTH = {0.64, 0.95, 2.53, 1.68, 0.8, 1.78, 2.42, 2.53, 0.03, 0.22, 1.78, 0.95, 0, 0.03, 0.8, 0.64}; // number: 21
	
	// from Lee2007 dinucleotide_features_news.xls
	public static double[] ROLL = {2.3, -2, 0.5, -8.1, 7.4, 1.4, 6.3, 0.5, 5, -0.4, 1.4, -2, 8.4, 5, 7.4, 2.3};
	
	// from Diprodb, confirmed by P0000002, and Rise
	// confirmed by Lee2007
	public static double[] RISE = {3.16, 3.41, 3.63, 3.89, 3.23, 4.08, 3.6, 3.63, 3.47, 3.81, 4.08, 3.41, 3.21, 3.47, 3.23, 3.16}; // number: 3
	
	// from Diprodb, confirmed by P0000033, and B-DNA Major groove size
	// confirmed by Lee2007
	public static double[] MAJOR_GROOVE_SIZE = {3.98, 3.98, 4.7, 4.7, 3.98, 3.98, 4.7, 4.7, 3.26, 3.26, 3.98, 3.98, 3.26, 3.26, 3.98, 3.98}; // number: 9

	// from Diprodb, confirmed by P0000031, and B-DNA Minor groove size
	// confirmed by Lee2007
	public static double[] MINOR_GROOVE_SIZE = {2.98, 3.26, 3.98, 3.26, 3.7, 3.98, 4.7, 3.98, 2.98, 3.26, 3.98, 3.26, 2.7, 2.98, 3.7, 2.98}; // number: 13
	
	// from Diprodb, confirmed by P0000024, and Mobility to bend toward major groove
	// confirmed by Lee2007
	public static double[] MAJOR_GROOVE_MOBILITY = {1.18, 1.06, 1.06, 1.12, 1.06, 0.99, 1.02, 1.04, 1.08, 0.98, 1, 1.02, 1.07, 1.03, 1.03, 1.09}; // number: 18

	// from Diprodb, confirmed by P0000025, and Mobility to bend toward minor groove
	// confirmed by Lee2007
	public static double[] MINOR_GROOVE_MOBILITY = {1.04, 1.1, 1.09, 1.02, 1.16, 1.27, 1.25, 1.16, 1.12, 1.17, 1.25, 1.11, 1.05, 1.2, 1.23, 1.04}; // number: 19
	
	// from Lee2007 dinucleotide_features_news.xls
	public static double[] SLIDE = {-0.03, -0.13, 0.47, -0.37, 1.46, 0.6, 0.63, 0.47, -0.07, 0.29, 0.6, -0.13, 0.74, -0.07, 1.46, -0.03};
	
	// from Diprodb
	public static double[] GC_CONTENT = {0, 1, 1, 0, 1, 2, 2, 1, 1, 2, 2, 1, 0, 1, 1, 0}; // number: 76
	
	// from Lee2007 dinucleotide_features_news.xls, DIPRODB 120
	// TODO the value for AC and AG are wrong, should be 35.8, 35.7
	public static double[] TWIST = {37.6, 35.8, 35.7, 39.7, 32.2, 35.5, 33.9, 35.7, 38.4, 37.4, 35.5, 35.8, 34.6, 38.4, 32.2, 37.6};
	
	// from Diprodb, confirmed by P0000023, and Probility to be contacting with a nucleosome core
	// confirmed by Lee2007
	public static double[] NUCLEOSOME_PROBABILITY = {18.4, 10.2, 14.5, 7.2, 15.7, 10.2, 1.1, 14.5, 11.3, 5.2, 10.2, 10.2, 6.2, 11.3, 15.7, 18.4}; // number: 17

	// from Diprodb, confirmed by P0000019, and B-DNA wedge
	// confirmed by Lee2007
	public static double[] WEDGE = {7.2, 1.1, 8.4, 2.6, 3.5, 2.1, 6.7, 8.4, 5.3, 5, 2.1, 1.1, 0.9, 5.3, 3.5, 7.2}; // number: 99
	
	// from Diprodb, confirmed by P0000038, and B-DNA free energy change
	// confirmed by Lee2007
	public static double[] FREE_ENERGY = {-1.2, -1.5, -1.5, -0.9, -1.7, -2.1, -2.8, -1.5, -1.5, -2.3, -2.1, -1.5, -0.9, -1.5, -1.7, -1.2}; // number: 72

	// from Lee2007 dinucleotide_features_news.xls
	public static double[] PERSISTENCE_LENGTH_1 = {50, 60, 60, 20, 60, 130, 85, 60, 60, 85, 130, 60, 20, 60, 60, 50};

	// from Lee2007 dinucleotide_features_news.xls
	public static double[] PERSISTENCE_LENGTH_2 = {37, 100, 43, 41, 38, 83, 63, 43, 84, 156, 83, 100, 11, 84, 38, 37};	

	/*
	 * The following are some misused properties, now they have no use.
	 * 
	 */
	// from Diprodb, confirmed by P0000001, and Twist
	public static double[] TWIST_D = {38.9, 31.12, 32.15, 33.81, 41.41, 34.96, 32.91, 32.15, 41.31, 38.5, 34.96, 31.12, 33.28, 41.31, 41.41, 38.9}; // number: 1
		
	// from Diprodb, confirmed by P0000013, and free DNA slide
	public static double[] SLIDE_D = {-0.1, -0.2, 0.4, -0.4, 1.6, 0.8, 0.7, 0.4, 0, 0.4, 0.8, -0.2, 0.9, 0, 1.6, -0.1}; // number: 96
	
	// from Diprodb, confirmed by P0000010, and Free DNA Roll
	public static double[] ROLL_D = {0.3, 0.5, 4.5, -0.8, 0.5, 6, 3.1, 4.5, -1.3, -6.2, 6, 0.5, 2.8, -1.3, 0.5, 0.3}; // number: 94	
	
	// from Diprodb, confirmed by P0000012, and free DNA tilt
	public static double[] TILT_D = {0.5, 0.1, 2.8, 0, -0.7, 2.7, 0, 2.8, 0.9, 0, 2.7, 0.1, 0, 0.9, -0.7, 0.5}; // number: 93
	
	// trinucleotide property
	public static double[] CPNPGCPGISLANDS = {0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 2, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 2, 1, 1, 0, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0};
	public static double[] CPNPGISLANDS = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 4, 0, 0, 4, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	
	// same as the one I found in the Hongkong's software
	public static double[] BENDABILITY = {-0.274, -0.205, -0.081, -0.28, -0.006, -0.032, -0.033, -0.183, 0.027, 0.017, -0.057, -0.183, 0.182, -0.110, 0.134, -0.280, 0.015, 0.040, 0.175, 0.134, -0.246, -0.012, -0.136, -0.057, -0.003, -0.077, -0.136, -0.033, 0.090, 0.031, 0.175, -0.081, -0.037, -0.013, 0.031, -0.110, 0.076, 0.107, -0.077, 0.017, 0.013, 0.107, -0.012, -0.032, 0.025, -0.013, 0.040, -0.205, 0.068, 0.025, 0.090, 0.182, 0.194, 0.013, -0.003, 0.027, 0.194, 0.076, -0.246, -0.006, 0.068, -0.037, 0.015, -0.274};
	public static double[] NUCLEOSOMEPOSITION = {-36, -6, 6, -30, 6, 8, 8, 11, -9, 25, 8, 11, -13, 7, 18, -30, -9, 17, -2, 18, 8, 13, 2, 8, 31, 25, 2, 8, -18, 8, -2, 6, -12, 8, 8, 7, 13, 45, 25, 25, -5, 45, 13, 8, -6, 8, 17, -6, -20, -6, -18, -13, 8, -5, 31, -9, 8, 13, 8, 6, -20, -12, -9, -36};
		
}
