/**
 * 
 */
package com.zhengyiyu.feature;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import com.zhengyiyu.common.Common;
import com.zhengyiyu.common.FolderPath;

/**
 * @author Yiyu
 *
 */
public class FeatureManager {
	public static HashMap<String, double[]> compositionFeatureMap;
	
	static {
		compositionFeatureMap = new HashMap<String, double[]>();
		compositionFeatureMap.put("TIP", DNAStructuralFeatureConversionTable.TIP);
		compositionFeatureMap.put("MINOR_GROOVE_MOBILITY", DNAStructuralFeatureConversionTable.MINOR_GROOVE_MOBILITY);
		compositionFeatureMap.put("TILT", DNAStructuralFeatureConversionTable.TILT);
		compositionFeatureMap.put("ZDNA_FREEENERGY", DNAStructuralFeatureConversionTable.ZDNA_FREEENERGY);
		compositionFeatureMap.put("PERSISTENCE_LENGTH_1", DNAStructuralFeatureConversionTable.PERSISTENCE_LENGTH_1);
		compositionFeatureMap.put("SLIDE", DNAStructuralFeatureConversionTable.SLIDE);
		compositionFeatureMap.put("MAJOR_GROOVE_MOBILITY", DNAStructuralFeatureConversionTable.MAJOR_GROOVE_MOBILITY);
		compositionFeatureMap.put("PROPELLERTWIST", DNAStructuralFeatureConversionTable.PROPELLERTWIST);
	}
	
	/**
	 * @return
	 * form Yeast Top10, Human Resting/Activated Top10, Peckham Top10, Lee StructuralFeatures
	 */
	public static ArrayList<SequenceFeature> getTopFeatures() {
		ArrayList<SequenceFeature> yeastTopFeatures = new ArrayList<SequenceFeature>();
		yeastTopFeatures.add(new DNAStructuralFeature("TIP", DNAStructuralFeatureConversionTable.TIP, "Lee2007 excel, Diprodb"));
		yeastTopFeatures.add(new DNAStructuralFeature("MINOR_GROOVE_MOBILITY", DNAStructuralFeatureConversionTable.MINOR_GROOVE_MOBILITY, "Lee2007 excel, Diprodb"));
		yeastTopFeatures.add(new DNAStructuralFeature("TILT", DNAStructuralFeatureConversionTable.TILT, "Lee2007 excel, Diprodb"));
		yeastTopFeatures.add(new DNAStructuralFeature("ZDNA_FREEENERGY", DNAStructuralFeatureConversionTable.ZDNA_FREEENERGY, "Lee2007 excel"));
		yeastTopFeatures.add(new DNAStructuralFeature("PERSISTENCE_LENGTH_1", DNAStructuralFeatureConversionTable.PERSISTENCE_LENGTH_1, "Lee2007 excel"));
		yeastTopFeatures.add(new DNAStructuralFeature("SLIDE", DNAStructuralFeatureConversionTable.SLIDE, "Lee2007 excel"));
		yeastTopFeatures.add(new DNAStructuralFeature("MAJOR_GROOVE_MOBILITY", DNAStructuralFeatureConversionTable.MAJOR_GROOVE_MOBILITY, "Lee2007 excel"));
		yeastTopFeatures.add(new DNAStructuralFeature("PROPELLERTWIST", DNAStructuralFeatureConversionTable.PROPELLERTWIST, "Lee2007 excel, Diprodb"));
		yeastTopFeatures.add(new CompositionFeature("ATA/TAT"));
		yeastTopFeatures.add(new CompositionFeature("ATAA/TTAT"));
		yeastTopFeatures.add(new CompositionFeature("AAT/ATT"));
		yeastTopFeatures.add(new CompositionFeature("ATTA/TAAT"));
		yeastTopFeatures.add(new CompositionFeature("TAA/TTA"));
		yeastTopFeatures.add(new CompositionFeature("TAATA/TATTA"));
		yeastTopFeatures.add(new CompositionFeature("AAAA/TTTT"));
		yeastTopFeatures.add(new CompositionFeature("CGCC/GGCG"));
		yeastTopFeatures.add(new CompositionFeature("AAG/CTT"));
		yeastTopFeatures.add(new CompositionFeature("ACA/TGT"));
		yeastTopFeatures.add(new CompositionFeature("AAATA/TATTT"));
		yeastTopFeatures.add(new CompositionFeature("CCGCC/GGCGG"));
		yeastTopFeatures.add(new CompositionFeature("GAC/GTC"));
		yeastTopFeatures.add(new CompositionFeature("CCCC/GGGG"));
		yeastTopFeatures.add(new CompositionFeature("ACAC/GTGT"));
		yeastTopFeatures.add(new CompositionFeature("AATTA/TAATT"));
		yeastTopFeatures.add(new CompositionFeature("ATAT"));
		yeastTopFeatures.add(new CompositionFeature("A/T"));
		yeastTopFeatures.add(new CompositionFeature("TA"));
		yeastTopFeatures.add(new CompositionFeature("AAA/TTT"));
		yeastTopFeatures.add(new CompositionFeature("AT"));
		yeastTopFeatures.add(new CompositionFeature("AATA/TATT"));
		return yeastTopFeatures;
	}

	public static ArrayList<SequenceFeature> restoreDiscretizedFeatures(String speciesName, String cellStatus, String rootFolder) throws IOException {
		
		ArrayList<SequenceFeature> discretizedFeatures = new ArrayList<SequenceFeature>();
		
		BufferedReader br = new BufferedReader(new FileReader(rootFolder + speciesName + cellStatus + "_discretization"));
		
		String line = "";
		String[] result = null;
		
		while ((line = br.readLine()) != null) {
			result = line.split(Common.SplitPattern_Tab);
			
			double[] cutoffs = new double[result.length - 3];
			for (int i = 3; i < result.length; i++) {
				cutoffs[i - 3] = Double.parseDouble(result[i]);
			}
			
			if (result[0].equals("DNAStructuralFeature")) {
				// DNAStructuralFeature	A	TIP	1.2586854738670774	1.475712096548739	1.6927387192304004
				DNAStructuralFeature f = new DNAStructuralFeature(result[2], compositionFeatureMap.get(result[2]), result[1], cutoffs);
				discretizedFeatures.add(f);
			} else if (result[0].equals("CompositionFeature")) {
				// CompositionFeature	e	AATA_TATT	0.003979707991783632	0.016836277521761395	0.029692847051739156
				CompositionFeature f = new CompositionFeature(result[2], result[1], cutoffs);
				discretizedFeatures.add(f);
			}
		}
		
		return discretizedFeatures;
	}

	public static void main(String[] args) throws IOException {
		String speciesName = "yeast";
		String cellStatus = "";
		
		
		ArrayList<SequenceFeature> features = restoreDiscretizedFeatures(speciesName, cellStatus, FolderPath.OUTPUT_FOLDER_Full + "pattern_" + speciesName  + cellStatus + File.separator);
		
		for (SequenceFeature sf : features) {
			for (Double d : sf.getDiscretizationCutoff()) {
				System.out.print("\t" + d);
			}
			
			System.out.println();
		}
	}
	
}
