package com.zhengyiyu.ffn;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.feature.FeatureManager;
import com.zhengyiyu.feature.SequenceFeature;
import com.zhengyiyu.handler.PatternHandler;
import com.zhengyiyu.obj.TransitionMatrix;

public class Step2_TransitionMatrix_Generation {
	
	public static void main(String[] args) throws IOException {
		String speciesName = "yeast";
		String cellStatus = "";
		
		if (args.length == 2) {
			speciesName = args[0];
			if (!speciesName.equals("yeast")) {
				cellStatus = args[1];
			} else {
				cellStatus = "";
			}
		}
		
		// some labels
		String nucleosomeName = "nucleosome";
		String linkerName = "linker";
		
		// begin to load data 
		System.out.println("###Species:" + speciesName + (cellStatus.equals("")? "" : ", cell status:" + cellStatus));
		
		String rootFolder = FolderPath.OUTPUT_FOLDER_Full + "pattern_" + speciesName + cellStatus + File.separator;
		String nucleosomeProfileName = rootFolder + speciesName + cellStatus + "_" + nucleosomeName + "_profile";
		String linkerProfileName = rootFolder + speciesName + cellStatus  + "_" + linkerName + "_profile";
		
		File nucleosomeProfileFile = new File(nucleosomeProfileName);
		if (nucleosomeProfileFile.exists() == false) {
			System.out.println("Profile Data does not exist!!! FilePath: " + nucleosomeProfileName);
			return;
		}
		
		ArrayList<SequenceFeature> discretizedFeatures = FeatureManager.restoreDiscretizedFeatures(speciesName, cellStatus, rootFolder);
		int featureSize = discretizedFeatures.size();
		
		System.out.println(featureSize);
		
		// nucleosome matrix
		
		ArrayList<double[][]> nucleosomeCountArray = PatternHandler.genereateBlankMatrix(discretizedFeatures, featureSize);
		ArrayList<int[]> nucleosomeProfile = PatternHandler.restoreProfileData(nucleosomeProfileName, nucleosomeCountArray, featureSize);
		ArrayList<TransitionMatrix> nucleosomeTransitionMatrix = PatternHandler.generateTransitionMatrix(nucleosomeCountArray, discretizedFeatures);
		String nucleosomeTransitionMatrixName = rootFolder + speciesName + cellStatus + "_" + nucleosomeName + "_transitionmatrix";
		PatternHandler.recordTransitionMatrix(nucleosomeTransitionMatrixName, nucleosomeTransitionMatrix);
		
		// linker matrix
		ArrayList<double[][]> linkerCountArray = PatternHandler.genereateBlankMatrix(discretizedFeatures, featureSize);
		ArrayList<int[]> linkerProfile = PatternHandler.restoreProfileData(linkerProfileName, linkerCountArray, featureSize);
		ArrayList<TransitionMatrix> linkerTransitionMatrix = PatternHandler.generateTransitionMatrix(linkerCountArray, discretizedFeatures);
		String linkerTransitionMatrixProfileName = rootFolder + speciesName + cellStatus + "_" + linkerName + "_transitionmatrix";
		PatternHandler.recordTransitionMatrix(linkerTransitionMatrixProfileName, linkerTransitionMatrix);
		
		System.out.println(nucleosomeProfile.size() + " : " + linkerProfile.size() + ";");
	}
}
