package com.zhengyiyu.ffn;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.feature.FeatureManager;
import com.zhengyiyu.feature.SequenceFeature;
import com.zhengyiyu.handler.PatternHandler;
import com.zhengyiyu.handler.ZScoreCalculator;
import com.zhengyiyu.obj.Pattern;
import com.zhengyiyu.obj.TransitionMatrix;
import com.zhengyiyu.util.FileUtil;

public class Step4_Pattern_DScore_Calculation {
	public static void main(String[] args) throws IOException {
		
		String speciesName = "yeast";
		String cellStatus = "";
		
		int startIndex = 0;
		int endIndex = 10000000;
		
		double alpha = 0.3;
		
		// parameters
		String caseName = "nucleosome";
		String controlName = "linker";
				
		if (args.length >= 5) {
			speciesName = args[0];
			if (!speciesName.equals("yeast")) {
				cellStatus = args[1];
			} else {
				cellStatus = "";
			}
			
			caseName = args[2];
			controlName = args[3];
			
			alpha = Double.parseDouble(args[4]);
			
			if (args.length >= 7) {
				startIndex = Integer.parseInt(args[5]);
				endIndex = Integer.parseInt(args[6]);
			}
		}
		
		double cutoff = 0.001;
		
		System.out.println("###Species:" + speciesName + (cellStatus.equals("") ? "" : ", cell status:" + cellStatus));
		
		String rootFolder = FolderPath.OUTPUT_FOLDER_Full + "pattern_" + speciesName + cellStatus + File.separator;

		ArrayList<SequenceFeature> discretizedFeatures = FeatureManager.restoreDiscretizedFeatures(speciesName, cellStatus, rootFolder);
		
		HashMap<String, ArrayList<TransitionMatrix>> transitionMatrixMaps = PatternHandler.restoreTransitionMatrixMap(speciesName, cellStatus, caseName, controlName, rootFolder);
		
		// restore all nucleosome and linkers to generate length related array, we do not need the detail info of nucleosome but only the length...
		HashMap<String, ArrayList<Integer>> lengthMap = new HashMap<String, ArrayList<Integer>>();
		HashMap<String, ArrayList<Integer>> noRepeatLengthMap = new HashMap<String, ArrayList<Integer>>();
		
		ArrayList<Integer> caselengths = PatternHandler.restoreLengthInfo(rootFolder, speciesName + cellStatus + "_" + caseName);
		ArrayList<Integer> controlLengths = PatternHandler.restoreLengthInfo(rootFolder, speciesName + cellStatus + "_" + controlName);
		
		System.out.println("###Total Number of case " + caseName + ":" + caselengths.size());
		System.out.println("###Total Number of control " + controlName + ":" + controlLengths.size());
		
		lengthMap.put(caseName, caselengths);
		lengthMap.put(controlName, controlLengths);
		
		noRepeatLengthMap.put(caseName, PatternHandler.getNoRepeatList(caselengths));
		noRepeatLengthMap.put(controlName, PatternHandler.getNoRepeatList(controlLengths));
		
		// to be quick, we start from length2 patterns
		String countFolder = rootFolder + "count" + alpha + File.separator;
		FileUtil.createFolder(countFolder);
		
		int length = 2;
		ArrayList<Pattern> allPatterns = new ArrayList<Pattern>();
		
		while (true) {
			length++;
			String countFile = countFolder + caseName + "_" + controlName + "_length" + length + "_count";
			
			if (new File(countFile).exists()) {
				ArrayList<Pattern> lengthPatterns = PatternHandler.restorePatternCountFile(countFile);
				allPatterns.addAll(lengthPatterns);
				
				System.out.println("###length" + length + " patterns size:" + lengthPatterns.size());
			} else {
				break;
			}
		}
		
		// calculate zscore
		int count = 0;
		
		ArrayList<Pattern> patternsFiltered = new ArrayList<Pattern>();
		System.out.println("###all patterns:" + allPatterns.size());
		
		patternsFiltered.addAll(allPatterns.subList(startIndex, Math.min(allPatterns.size(), endIndex)));
		
		System.out.println("### start:" + startIndex + "\t" + patternsFiltered.get(0));
		System.out.println("### end:" + endIndex + "\t" + patternsFiltered.get(patternsFiltered.size() - 1));
		System.out.println("### patterns to calculate: " + patternsFiltered.size());
		
		for (Pattern pattern : patternsFiltered) {
			
			int caseCount = (Integer) pattern.getInfo().get(caseName + "_count");
			System.out.println(count++ + "\t" + pattern);
			System.out.println("calculating case :");
			
			double caseZScore = ZScoreCalculator.calculateZScore(transitionMatrixMaps.get(caseName), lengthMap.get(caseName), noRepeatLengthMap.get(caseName), 
					pattern, caseCount, caseName, cutoff); 
			
			int controlCount = (Integer) pattern.getInfo().get(controlName + "_count");
			System.out.println("calculating control :");
			double controlZScore = ZScoreCalculator.calculateZScore(transitionMatrixMaps.get(controlName), lengthMap.get(controlName), noRepeatLengthMap.get(controlName), 
					pattern, controlCount, controlName, cutoff); 
			
			double patternScore = caseZScore - controlZScore;
			
			pattern.getInfo().put(caseName + "_" + controlName, patternScore);
			pattern.setPatternScore(patternScore);
			
			System.out.println(pattern + "_logratio: " + patternScore);
			System.out.println();
		}
		
		// rank the pattern
		Collections.sort(patternsFiltered, PatternHandler.getPatternScoreComparator());
		Collections.reverse(patternsFiltered);
		
		// record the result
		String zscoreRootFolder = rootFolder + "zscore" + alpha + File.separator;
		FileUtil.createFolder(zscoreRootFolder);
		
		ZScoreCalculator.recordZScore(zscoreRootFolder + caseName + "_" + controlName + "_" + startIndex + "_" + endIndex, 
				patternsFiltered, discretizedFeatures, caseName, controlName);
		
	}
}
