package com.zhengyiyu.handler;

import java.util.ArrayList;
import java.util.HashMap;

import com.zhengyiyu.util.DoubleUtil;

/**
 * @author Yiyu
 * Utility class for nucleotide, especially mononucleotide and dinucleotide
 */
public class DiNucleotideHandler {
	// AA	AC	AG	AT	CA	CC	CG	CT	GA	GC	GG	GT	TA	TC	TG	TT
	public static String[] MONONUCLEOTIDE_ARRAY = {"A", "C", "G", "T"};
	
	public static String[] DINUCLEOTIDE_ARRAY = {"AA", "AC", "AG", "AT", "CA", "CC", "CG", "CT", "GA", "GC", "GG", "GT", "TA", "TC", "TG", "TT"};
	
	/**
	 * the map contain all the combination of dinucleotide including the lowercase, uppercase and mixed.
	 */
	public static HashMap<String, Integer> DINUCLEOTIDE_MAP = new HashMap<String, Integer>();
	
	static {
		// generate all kinds of ACGT
		ArrayList<String> ACGT_LIST = new ArrayList<String>();
		ACGT_LIST.add("A");
		ACGT_LIST.add("C");
		ACGT_LIST.add("G");
		ACGT_LIST.add("T");
		
		ArrayList<String> DINUCLEOTIDE_LIST = new ArrayList<String>();
		for (String s1 : ACGT_LIST) {
			for (String s2 : ACGT_LIST) {
				DINUCLEOTIDE_LIST.add(s1 + s2);
			}
		}
		
		// DINUCLEOTIDE_LIST will like DINUCLEOTIDE_ARRAY
		ACGT_LIST.add("a");
		ACGT_LIST.add("c");
		ACGT_LIST.add("g");
		ACGT_LIST.add("t");

		String dn = "";
		
		for (String s1 : ACGT_LIST) {
			for (String s2 : ACGT_LIST) {
				dn = s1 + s2;
				DINUCLEOTIDE_MAP.put(dn, DINUCLEOTIDE_LIST.indexOf(dn.toUpperCase()));
			}
		}
	}
	
	/**
	 * @param conversionTable
	 * @param sequence
	 * @return the profile array of the given sequence based on the dinucleotide conversion table
	 */
	public static ArrayList<Double> calculateDinucleotidePropertyProfile(double[] conversionTable, String sequence) {
		ArrayList<Double> profile = new ArrayList<Double>();
		if (sequence == null || sequence.length() < 2) {
			return profile;
		}
		
		char firstLetter = sequence.charAt(0);
		
		for (int i = 1; i < sequence.length(); i++) {
			char secondLetter = sequence.charAt(i);
			Integer index = DINUCLEOTIDE_MAP.get("" + firstLetter + secondLetter);
			
			if (index == null) {
				// not find ... so just put a 0.0
				profile.add(DoubleUtil.NULL_DOUBLE);
			} else {
				profile.add(conversionTable[index]);
			}
			firstLetter = secondLetter;
		}
		return profile;
	}
}
