package com.zhengyiyu.handler;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import com.zhengyiyu.common.Common;
import com.zhengyiyu.obj.Gene;
import com.zhengyiyu.obj.Nucleosome;
import com.zhengyiyu.obj.Strand;
import com.zhengyiyu.util.RegionUtil;
import com.zhengyiyu.util.StringUtil;

/**
 * @author Yiyu Zheng
 *
 */
public class NucleosomeDataHandler {
	
	/**
	 * Well Known Nucleosome length is 147 bp
	 */
	public static int NucleosomeLength = 147;
	
	/**
	 * @param nsandlinkers
	 * Set the neighbour nucleosome / linker for the each nucleosome and linker in the list
	 */
	public static void generateNeigbor(ArrayList<Nucleosome> nsandlinkers) {
		for (int i = 0; i < nsandlinkers.size(); i++) {
			Nucleosome ns = nsandlinkers.get(i);
			
			if (ns.isLinker() == false) {
				if (i > 0) {
					if (nsandlinkers.get(i - 1).isLinker()) {
						// to avert some nucleosomes are just continuel without a linker
						ns.getInfo().put("left_linker", nsandlinkers.get(i - 1));
					}
				}
				
				if (i < nsandlinkers.size() - 1) {
					if (nsandlinkers.get(i + 1).isLinker()) {
						ns.getInfo().put("right_linker", nsandlinkers.get(i + 1));
					}
				}
			} else {
				if (i > 0) {
					if (!nsandlinkers.get(i - 1).isLinker()) {
						ns.getInfo().put("left_nucleosome", nsandlinkers.get(i - 1));
					}	
				}
				
				if (i < nsandlinkers.size() - 1) {
					if (!nsandlinkers.get(i + 1).isLinker()) {
						ns.getInfo().put("right_nucleosome", nsandlinkers.get(i + 1));
					}
				}
			}
		}
	}

	/**
	 * @param rootFolder
	 * @param speciesName
	 * @param cellStatus
	 * @param allowNucleosomeInMargin
	 * @param promoterRegion
	 * @param broaderRegion
	 * @return
	 * @throws IOException
	 * 
	 * extract nucleosome, gene, transcript information for data file
	 */
	public static ArrayList<Gene> restoreGeneTranscriptNucleosome(String rootFolder, String speciesName,
			String cellStatus, boolean allowNucleosomeInMargin, int[] promoterRegion, int[] broaderRegion) throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(rootFolder + speciesName + cellStatus + "_" + "geneSeq" + "_" + broaderRegion[0] + "_" + broaderRegion[1]));
		String line = "";
		String[] result = null;
		
		ArrayList<Gene> genes = new ArrayList<Gene>();
		
		while ((line = br.readLine()) != null) {
			if (line.trim().equals("")) {
				// just in case a blank line somewhere
				continue;
			}
			
			System.out.println(line);
			result = line.split(Common.SplitPattern_Blank);
			
			Gene g = new Gene(StringUtil.convertChrCompact2SequencePostion(result[1]));
			g.setStrand(Strand.getStrand(result[2]));
			g.setGeneId(result[3]);
			g.setGeneName(result[4]);
			
			for (int i = 5; i < result.length; i++) {
				g.getExps().add(Double.parseDouble(result[i]));
			}
			
			line = br.readLine();
			int pos = line.indexOf(",");
			g.getInfo().put("seqstart", Integer.parseInt(StringUtil.subString(line, line.indexOf("[") + 1, pos)));
			g.getInfo().put("seqend", Integer.parseInt(StringUtil.subString(line, pos + 2, line.length() - 1)));
			
			line = br.readLine();
			g.getInfo().put("seq", line);
			
			// need to only get the nucleosome and linkers in the region...
			int[] region = RegionUtil.getPromoterRegion(g, promoterRegion);
			
			ArrayList<Nucleosome> allNsandlinkers = new ArrayList<Nucleosome>();
			
			// maybe first all, find neibour, and then...
			while ((line = br.readLine()) != null && !line.trim().equals("")) {
				result = line.split(Common.SplitPattern_Blank);
				Nucleosome ns = new Nucleosome(StringUtil.convertChrCompact2SequencePostion(result[0]), Integer.parseInt(result[3]));
				ns.getInfo().put("averagescore", Double.parseDouble(result[4]));
				
				if (speciesName.equals("human") && ns.getStatus() == 1) {
					// this is a delocalized with a new dyad
					ns.setMiddlePosition(Integer.parseInt(result[5]));
				}
				allNsandlinkers.add(ns);
			}
			NucleosomeDataHandler.generateNeigbor(allNsandlinkers);
			
			ArrayList<Nucleosome> nsandlinkers = new ArrayList<Nucleosome>();
			
			for (Nucleosome ns : allNsandlinkers) {
				if (ns.isOverlapWithBody(region[0], region[1])) {
					nsandlinkers.add(ns);
				}
			}
			
			if (nsandlinkers.size() == 0) {
				// this time should not happne
				
				System.out.println("No way, this time gene cannot have no ns and linker");
				System.out.println(g);
				continue;
			}
			
			if (nsandlinkers.size() == 1) {
				if (nsandlinkers.get(0).getStatus() == 0 || nsandlinkers.get(0).getStatus() == 3) {
					// if only linkers, then we discard it!
					continue;
				}
			}
			
			// if there is no nucleosome/ only one linker in this region, we just do not include it.
			g.getInfo().put("nsandlinkers", nsandlinkers);
			genes.add(g);
		}
		
		return genes;
	}

}
