package com.zhengyiyu.handler;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;

import com.zhengyiyu.common.Common;
import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.feature.DNAStructuralFeature;
import com.zhengyiyu.feature.SequenceFeature;
import com.zhengyiyu.obj.Gene;
import com.zhengyiyu.obj.Pattern;
import com.zhengyiyu.obj.TransitionMatrix;
import com.zhengyiyu.util.ArrayUtil;

public class PatternHandler {
	public static final double patternScoreThreshold = 3 * Math.sqrt(2);
	
	public static final double notExistFeaturesThreshold = 1.0 / 146;
	
	public static ArrayList<SequenceFeature> attrs = null;
	
	public static ArrayList<double[][]> genereateBlankMatrix(ArrayList<SequenceFeature> attrs, int attrSize) {
		ArrayList<double[][]> matrixList = new ArrayList<double[][]>();
		for (int i = 0; i < attrSize; i++) {
			int discretizeSize = attrs.get(i).getDiscretizationCutoff().size() + 1;
			double[][] matrix = new double[discretizeSize][discretizeSize];
			for (int a = 0; a < discretizeSize; a++) {
				for (int b = 0; b < discretizeSize; b++) {
					matrix[a][b] = 0.0;
				}
			}
			matrixList.add(matrix);
		}
		
		return matrixList;
	}
	
	/**
	 * restore the arraylist from the profile data and also generate the count transitionMatrix
	 * @param profileDataFileName
	 * @param transitionMatrix
	 * @param featureSize
	 * @return
	 * @throws IOException
	 */
	public static ArrayList<int[]> restoreProfileData(String profileDataFileName, ArrayList<double[][]> transitionMatrix, int featureSize) throws IOException {
		ArrayList<int[]> data = new ArrayList<int[]>();
		
		BufferedReader br = new BufferedReader(new FileReader(profileDataFileName));
		
		String line = "";
		String[] result = null;
		
		int[] lastData = null;
		
		while((line = br.readLine()) != null) {
			if (line.startsWith("###")){
				lastData = null;
				continue;
			} else {
				result = line.trim().split(Common.SplitPattern_Tab);
				
				int[] d = new int[result.length];
				for (int i = 0; i < result.length; i++) {
					d[i] = Integer.parseInt(result[i]);
				}
				
				// read a profile, and add it to the arraylist
				data.add(d);
				
				// record the transition
				if (transitionMatrix!= null && lastData != null) {
					for (int featureIndex = 0; featureIndex < featureSize; featureIndex++) {
						// if last time it is 1 and this time it is 2, then set the a1a2++;
						transitionMatrix.get(featureIndex)[lastData[featureIndex]][d[featureIndex]]++;
					}
				}
				lastData = Arrays.copyOf(d, d.length);
				
				if( data.size() % 1000000 == 0) {
					 System.out.println("###profile data loaded:" + data.size());
				}
			}
		}
		
		System.out.println("###Data total Size:" + data.size());
		
		br.close();
		return data;
	}
	
	public static ArrayList<TransitionMatrix> generateTransitionMatrix(ArrayList<double[][]> nucleosomeMatrix, ArrayList<SequenceFeature> discretizedFeatures) {
		ArrayList<TransitionMatrix> matrixList = new ArrayList<TransitionMatrix>();
		
		for (int i = 0; i < discretizedFeatures.size(); i++) {
			TransitionMatrix tm = generateTransitionMatrix(nucleosomeMatrix.get(i), discretizedFeatures.get(i));
			matrixList.add(tm);
		}
		
		return matrixList;
	}
	
	private static TransitionMatrix generateTransitionMatrix(double[][] countMatrix, SequenceFeature feature) {
		int matrixSize = countMatrix.length;
		
		System.out.println("###Feature:" + feature.getFeatureName());
		
		TransitionMatrix tm = new TransitionMatrix(feature.getPatternName(), feature.getFeatureName());
		tm.setCountMatrix(countMatrix);
		ArrayUtil.printDoubleArray("Orignal Count Matrix", countMatrix);
		
		// get percentage
		double[][] matrix = new double[matrixSize][matrixSize];
		for (int i = 0; i < matrixSize; i++) {
			for (int j = 0; j < matrixSize; j++) {
				matrix[i][j] = countMatrix[i][j];
			}
		}
		
		for (int a = 0; a < matrixSize; a++) {
			double[] subMatrix = matrix[a];
			
			double sum = 0.0;
			for (int b = 0; b < matrixSize; b++) {
				sum += subMatrix[b];
			} 
			
			if (sum > 0) {
				for (int b = 0; b < matrixSize; b++) {
					subMatrix[b] = subMatrix[b] / sum;
				} 
			}
		}
		
		
		tm.setMatrix(matrix);
		
		tm.transformMatrix();
		return tm;
	}
	
	public static void recordTransitionMatrix(String transitionMatrixName, ArrayList<TransitionMatrix> matrixes) throws IOException {
		BufferedWriter bw = new BufferedWriter(new FileWriter(transitionMatrixName));
		for (TransitionMatrix tm : matrixes) {
			bw.write(tm.getLabel() + "\t" + tm.getName());
			bw.newLine();
			bw.write("Count Matrix");
			bw.newLine();
			for (double[] rowData : tm.getCountMatrix()) {
				for (Double d : rowData) {
					bw.write("\t" + d);
				}
				bw.newLine();
			}
			
			bw.write("Transition Matrix");
			bw.newLine();
			for (double[] rowData : tm.getMatrix()) {
				for (Double d : rowData) {
					bw.write("\t" + d);
				}
				bw.newLine();
			}
			
			bw.write("Pi Vector");
			bw.newLine();
			
			for (double d : tm.getPi()) {
				bw.write("\t" + d);
			}
			bw.newLine();
		}
		
		bw.close();
	}
	
	public static Integer getProfileSizeNoTandemRepeat(String species, String cellStatus, String caseName) {
		if (species.equals("yeast")) {
			return caseName.equals("nucleosome") ? 1096274 : 839093;
		} else if (species.equals("human")) {
			if (cellStatus.equals(NucleosomeDataHandler_Human.Cell_Status_Resting)) {
				return caseName.equals("nucleosome") ? 4291478 : 769615;
			} else if (cellStatus.equals(NucleosomeDataHandler_Human.Cell_Status_Activated)) {
				return caseName.equals("nucleosome") ? 4106580 : 802672;
			}
		}
		return null;
	}
	
	public static Integer getProfileSize(String species, String cellStatus, String caseName) {
		if (species.equals("yeast")) {
			return caseName.equals("nucleosome") ? 1096274 : 839093;
		} else if (species.equals("human")) {
			if (cellStatus.equals(NucleosomeDataHandler_Human.Cell_Status_Resting)) {
				return caseName.equals("nucleosome") ? 3265840 : 1919064;
			} else if (cellStatus.equals(NucleosomeDataHandler_Human.Cell_Status_Activated)) {
				return caseName.equals("nucleosome") ? 3239370 : 1913574;
			}
		}
		return null;
	}
	
	public static HashMap<String, ArrayList<TransitionMatrix>> restoreTransitionMatrixMap(String speciesName, String cellStatus, String caseName, String controlName, String rootFolder) throws IOException {
		ArrayList<TransitionMatrix> nucleosomeTransitionMatrix = restoreTransitionMatrix(rootFolder + speciesName + cellStatus + "_" +  caseName + "_transitionmatrix" );
		ArrayList<TransitionMatrix> linkerTransitionMatrix = restoreTransitionMatrix(rootFolder + speciesName + cellStatus + "_" + controlName + "_transitionmatrix");
	
		HashMap<String, ArrayList<TransitionMatrix>> transitionMatrixMaps = new HashMap<String, ArrayList<TransitionMatrix>>();
		transitionMatrixMaps.put(caseName, nucleosomeTransitionMatrix);
		transitionMatrixMaps.put(controlName, linkerTransitionMatrix);
		
		return transitionMatrixMaps;
	}
	
	public static ArrayList<TransitionMatrix> restoreTransitionMatrix(String nucleosomeTransitionMatrixName) throws IOException {
		ArrayList<TransitionMatrix> transitionMatrices = new ArrayList<TransitionMatrix>();
		String line = "";
		String[] result = null;
		BufferedReader br = new BufferedReader(new FileReader(nucleosomeTransitionMatrixName));
		while ((line = br.readLine()) != null) {
			result = line.split(Common.SplitPattern_Blank);
			TransitionMatrix transitionMatrix = new TransitionMatrix(result[0], result[1]);
			
			br.readLine(); // just the Count Matrix
			double[][] countMatrix = new double[4][4];
			// read next four lines to get the count matrix
			
			for (int row = 0; row < 4; row++) {
				line = br.readLine();
				
				result = line.trim().split(Common.SplitPattern_Blank);
				
				for (int column = 0; column <4; column++) {
					countMatrix[row][column] = Double.parseDouble(result[column]);
				}
			}
			
			transitionMatrix.setCountMatrix(countMatrix);
			
			br.readLine(); // the markov chain Matrix
			double[][] matrix = new double[4][4];
			// read next four lines to get the count matrix
			
			for (int row = 0; row < 4; row++) {
				line = br.readLine();
				
				result = line.trim().split(Common.SplitPattern_Blank);
				
				for (int column = 0; column <4; column++) {
					matrix[row][column] = Double.parseDouble(result[column]);
				}
			}
			
			transitionMatrix.setMatrix(matrix);
			
			br.readLine();
			line = br.readLine();
			result = line.trim().split(Common.SplitPattern_Blank);
			
			double[] pi = new double[4];
			// read next four lines to get the count matrix
			
			for (int row = 0; row < 4; row++) {
				pi[row] = Double.parseDouble(result[row]);
			}
			
			transitionMatrix.setPi(pi);
			
			transitionMatrices.add(transitionMatrix);
		}
		
		return transitionMatrices;
	}
	
	public static ArrayList<Integer> restoreLengthInfo(String rootFolder, String prefix) throws IOException {
		// nucleosome_list_nsonly
		BufferedReader br = new BufferedReader(new FileReader(rootFolder + prefix + "_list"));
		ArrayList<Integer> lengthList = new ArrayList<Integer>();
		
		String line = "";
		String[] result = null;
		
		// ###	chr1:30644-30790	30717	147	2	197
		while ((line = br.readLine()) != null) {
			result = line.split(Common.SplitPattern_Tab);
			lengthList.add(Integer.parseInt(result[5]));
		}
		
		return lengthList;
	}
	

	public static ArrayList<Integer> getNoRepeatList(ArrayList<Integer> lengths) {
		HashSet<Integer> lengthSet = new HashSet<Integer>();
		
		for (Integer i : lengths) {
			lengthSet.add(i);
		}
		
		ArrayList<Integer> noRepeatLengthList = new ArrayList<Integer>();
		
		for (int i = 0; i < lengthSet.toArray().length; i++) {
			noRepeatLengthList.add((Integer) lengthSet.toArray()[i]);
		}
		
		Collections.sort(noRepeatLengthList);
		return noRepeatLengthList;
	}

	public static ArrayList<Pattern> countLength2Patterns(int beta, ArrayList<SequenceFeature> attrs, ArrayList<int[]> profileData, String info) throws IOException {
		ArrayList<Pattern> patterns = PatternHandler.generateLengthLength2Patterns(attrs);
		
		System.out.println("###Length" + 2 + "_pattern_size:" + patterns.size());
		
		return PatternHandler.countPatternNumber(patterns, beta, profileData, info);
		
	}
	
	public static String translatePattern(ArrayList<SequenceFeature> attrs, Pattern d) {
		StringBuffer sb = new StringBuffer();
		for (int featureIndex : d.getSubFeatures()) {
			sb.append(attrs.get(featureIndex).getFeatureName() + ",");
		}
		
		if (sb.toString().length() > 0) {
			sb.deleteCharAt(sb.length() - 1);
		}
		return sb.toString();
	}
	
	public static ArrayList<Pattern> countPatternNumber(ArrayList<Pattern> patterns, int beta, ArrayList<int[]> profileData, String countType) {
		ArrayList<Pattern> filteredPatterns = new ArrayList<Pattern>();
		
		// count dimer number
		int index = 0;
		for (Pattern d : patterns) {
			int count = countPattern(profileData, d);
			
			index++;
			if (index % 500 == 0) {
				System.out.println("Counting pattern num:" + d.toString() + "\t" + count + "\t" + index);
			}
			
//			System.out.println("Counting pattern num:" + d.toString() + "\t" + count + "\t" + index + "\t" + (count >= beta));
			if (count >= beta) {
				d.setCount(count);
				d.getInfo().put(countType, count);
				filteredPatterns.add(d);
			}
		}
		return filteredPatterns;
	}
	
	public static int countPattern(ArrayList<int[]> data, Pattern d) {
		int count = 0;
		for (int[] dt : data) {
			boolean good = true;
			for (int i = 0; i < d.getSubFeatures().size(); i++) {
				if (dt[d.getSubFeatures().get(i)] != d.getSubIndexs().get(i)) {
					good = false;
					break;
				}
			}
			if (good == true) {
				count++;
			}
		}
		return count;
	}

	private static ArrayList<Pattern> generateLengthLength2Patterns(ArrayList<SequenceFeature> attrs) {
		ArrayList<Pattern> length2Patterns = new ArrayList<Pattern>();
		int attrSize = attrs.size();
		for (int a = 0; a < attrSize; a++) {
			SequenceFeature attrA = attrs.get(a);
			int aDiscretizeSize = attrA.getDiscretizationCutoff().size() + 1;
			
			for (int asub = 0; asub < aDiscretizeSize; asub++) {
				for (int b = 0; b < attrSize; b++) {
					SequenceFeature attrB = attrs.get(b);
					if (b <= a) {
						continue;
					}
					
					int bDiscretizeSize = attrB.getDiscretizationCutoff().size() + 1;
					for (int bsub = 0; bsub < bDiscretizeSize; bsub++) {
						Pattern p = new Pattern(attrA.getPatternName() + asub + attrB.getPatternName() + bsub, 2);
						length2Patterns.add(p);
					}
				}
			}
		}
		return length2Patterns;
	}
	
	public static Comparator<Pattern> getCountComparator() {
		return new Comparator<Pattern>() {
			@Override
			public int compare(Pattern o1, Pattern o2) {
				return o1.getCount() - o2.getCount();
			}
		};
	}
	
	public static ArrayList<Pattern> combineDimers(ArrayList<Pattern> dimers, ArrayList<SequenceFeature> discretizedFeatures) {
		HashSet<String> existedDimerSet = new HashSet<String>();
		for (Pattern d : dimers) {
			existedDimerSet.add(d.toString());
		}
		
		ArrayList<Pattern> dimerList = new ArrayList<Pattern>();
		for (int i = 0; i < dimers.size(); i++) {
			if (i % 500 == 0) {
				System.out.println("Combine Dimers, current i = " + i);
			}
			
			Pattern currentDimer = dimers.get(i);
			
			for (SequenceFeature feature : discretizedFeatures) {
				if (currentDimer.toString().contains(feature.getPatternName())) {
					continue; // this pattern already exist
				}
				
				for (int j = 0; j < feature.getDiscretizationCutoff().size(); j++) {
					ArrayList<String> sub = new ArrayList<String>();
					sub.addAll(currentDimer.getSub());
					sub.add(feature.getPatternName() + j + "");
					
					boolean qualify = true;
					
					// if all exist, then keep, otherwise, discard
					Collections.sort(sub);
					
					for (int index = 0; index < sub.size(); index++) {
						String test = "";
						
						for (int jumpIndex = 0; jumpIndex < sub.size(); jumpIndex++) {
							if (jumpIndex != index) {
								test += sub.get(jumpIndex);
							}
						}
						
						if (!existedDimerSet.contains(test)) {
							qualify = false;
							break;
						}
					}
					
					if (qualify == true) {
						Pattern d = new Pattern(sub);
						
						if (!dimerList.contains(d)) {
							dimerList.add(d);
						}
					}
				}
			}
			
		}	
		
		Collections.sort(dimerList);
		return dimerList;
	}
	
	public static void recordFilterLengthPatterns(String fileName, ArrayList<Pattern> patterns, ArrayList<SequenceFeature> attrs, String caseName, String controlName) throws IOException {
		BufferedWriter bw = new BufferedWriter(new FileWriter(fileName));
		bw.write("###Pattern\tDetail\t" + caseName + "_count\t" + controlName + "_count\tindex");
		bw.newLine();
		int index = 0;
		for (Pattern d : patterns) {
			bw.write(d.toString() + "\t" + translatePattern(attrs, d) + "\t" + d.getInfo().get(caseName + "_count") + "\t" + d.getInfo().get(controlName + "_count") + "\t" + index++);
			bw.newLine();
		}
		bw.close();
	}

	public static ArrayList<Pattern> restorePatternCountFile(String countFile) throws IOException {
		
		ArrayList<Pattern> patterns= new ArrayList<Pattern>();
		BufferedReader br = new BufferedReader(new FileReader(countFile));
		String line = "";
		String[] result = null;
		
		line = br.readLine();
		result = line.split(Common.SplitPattern_Tab);
		
		String caseType = result[2];
		String controlType = result[3];
		
		while ((line = br.readLine())!= null) {
			if (line.startsWith("###") || line.trim().equals("")) {
				continue;
			}
			
//			###Pattern	Detail	nucleosome_count	linker_count	index
//			A0B0	TIP,MINOR_GROOVE_MOBILITY	null	null	0
//			A0B1	TIP,MINOR_GROOVE_MOBILITY	null	null	1
			result = line.split(Common.SplitPattern_Tab);
			Pattern d = new Pattern(result[0], 2);
			d.setCount(Integer.parseInt(result[2]));
			
			d.getInfo().put(caseType, Integer.parseInt(result[2]));
			d.getInfo().put(controlType, Integer.parseInt(result[3]));
			
			patterns.add(d);
		}
		
		br.close();
		return patterns;
	}

	public static ArrayList<Pattern> restoreAllPatternScoreFile(String zscoreFolder, String caseName, String controlName) throws IOException {
		ArrayList<Pattern> allPatterns = new ArrayList<Pattern>();
		
		int length = 3;
		
		while (true) {
			// nucleosome_linker_length3_all
			File f = new File(zscoreFolder + caseName + "_" + controlName + "_length" + length);
			
			if (f.exists()) {
				ArrayList<Pattern> lengthPatterns = restoreLengthPatternScore(f, caseName, controlName);
				System.out.println("###length" + length + " patterns size:" + lengthPatterns.size());
				allPatterns.addAll(lengthPatterns);

				length++;
			} else {
				break;
			}
		}
		
		System.out.println("###total pattern number:" + allPatterns.size());
		System.out.println();
		return allPatterns;
	}
	
	public static ArrayList<Pattern> restoreAllPatternScoreFile(String zscoreFolder, String caseName, String controlName, int maxPatternLength, int minPatternLength) throws IOException {
		PatternHandler.transformFeatureName();
		
		ArrayList<Pattern> allPatterns = new ArrayList<Pattern>();
		
		int length = minPatternLength;
		
		while (length <= maxPatternLength) {
			// nucleosome_linker_length3_all
			File f = new File(zscoreFolder + caseName + "_" + controlName + "_length" + length);
			
			if (f.exists()) {
				ArrayList<Pattern> lengthPatterns = restoreLengthPatternScore(f, caseName, controlName);
				System.out.println("###length" + length + " patterns size:" + lengthPatterns.size());
				allPatterns.addAll(lengthPatterns);

				length++;
			} else {
				break;
			}
		}
		
		System.out.println("###total pattern number:" + allPatterns.size());
		System.out.println();
		return allPatterns;
	}

	public static void transformFeatureName() {
		for (SequenceFeature feature : attrs) {
			if (feature instanceof DNAStructuralFeature) {
				String name = feature.getFeatureName().replaceAll("_", " ").toLowerCase();
				if (name.equals("zdna freeenergy")) {
					name = "Z-DNA free energy";
				} else if (name.equals("")) {
					name = "";
				}
				
				feature.setFeatureName(name);
			} else{
				feature.setFeatureName(feature.getFeatureName().replaceAll("_", "/").toUpperCase());
			}
		}
		
	}

	private static ArrayList<Pattern> restoreLengthPatternScore (File scoreFile, String caseName, String controlName) throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(scoreFile));
		String line = "";
		String[] result = null;
		
		ArrayList<Pattern> lengthPatterns = new ArrayList<Pattern>();
		
		while ((line = br.readLine()) != null) {
			if (line.startsWith("###")) {
				continue;
			}
			
			// T0V0W0	CCGCC_GGCGG,CCCC_GGGG,ACAC_GTGT	NaN	1.0847885169499092	NaN	269726	340644	0
			result = line.split(Common.SplitPattern_Tab);
			
			if (result[2].equals("NaN") || result[3].equals("NaN") || result[4].equals("NaN")) {
				continue;
			}
			
			
			Pattern p = new Pattern(result[0], 2);
			
			p.setPatternDetail(genereatePatternDetail(p));
			
			p.setPatternScore(Double.parseDouble(result[2]));
			p.getInfo().put(caseName + "_zscore", Double.parseDouble(result[3]));
			p.getInfo().put(controlName + "_zscore", Double.parseDouble(result[4]));
			
			p.getInfo().put(caseName + "_count", Integer.parseInt(result[5]));
			p.getInfo().put(controlName + "_count", Integer.parseInt(result[6]));
			
			if (p.getPatternScore() >= patternScoreThreshold) {
				
				if (scoreFile.getAbsolutePath().contains("human")) {
					if (isBadPattern(p)) {
						// System.out.println("Bad Pattern:"+ p);
						continue;
					}
			    }
				
//				if (isBadPatternPrecise(p)) {
//					System.out.println("Precise Bad pattern:" + p);
//					count++;
//					continue;
//				}
				
				lengthPatterns.add(p);
			}
			
		}
		
		
		return lengthPatterns;
	}
	
	public static String genereatePatternDetail(Pattern p) {
		if (attrs == null) {
			return "";
		} else {
			StringBuffer sb = new StringBuffer();
			for (int i = 0; i < p.getSub().size(); i++) {
				sb.append("\"" + attrs.get(p.getSubFeatures().get(i)).getFeatureName() + "\" level " + p.getSubIndexs().get(i) + ", ");
			}
			
			if (sb.length() > 2) {
				return sb.toString().substring(0, sb.toString().length() - 2);
			} else {
				return sb.toString();
			}
		}
		
	}

	private static boolean isBadPattern(Pattern p) {
		int count = 0;
		
		for (int i = 0; i < p.getSubIndexs().size(); i++) {
			if (p.getSubFeatures().get(i) < 8 || p.getSubIndexs().get(i) > 0) {
				count++;
			}
		}
		
		return count <= 1;
	}
	
	public static boolean isBadPatternPrecise(Pattern p) {
		int count = 0;
		
		for (int i = 0; i < p.getSubIndexs().size(); i++) {
			if (p.getSubFeatures().get(i) < 8 // structural features
					|| p.getSubIndexs().get(i) > 0 // not in level 0
					|| attrs.get(p.getSubFeatures().get(i)).getDiscretizationCutoff().get(0) > notExistFeaturesThreshold) {
				count++;
			}
		}
		
		return count <= 1;
	}
	
	public static void main(String[] args) {
		System.out.println(isBadPattern(new Pattern("F0G0H0", 2)));
		System.out.println(isBadPattern(new Pattern("F0H0I0", 2)));
		System.out.println(isBadPattern(new Pattern("I0J0K0", 2)));
		System.out.println(isBadPattern(new Pattern("I1J1K1", 2)));
	}

	public static ArrayList<Pattern> restoreLengthPatternScoreAll (File scoreFile, String caseName, String controlName) throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(scoreFile));
		String line = "";
		String[] result = null;
		
		ArrayList<Pattern> lengthPatterns = new ArrayList<Pattern>();
		
		while ((line = br.readLine()) != null) {
			if (line.startsWith("###")) {
				continue;
			}
			
			// T0V0W0	CCGCC_GGCGG,CCCC_GGGG,ACAC_GTGT	NaN	1.0847885169499092	NaN	269726	340644	0
			result = line.split(Common.SplitPattern_Tab);
			
			if (result[2].equals("NaN") || result[3].equals("NaN") || result[4].equals("NaN")) {
				continue;
			}
			
			Pattern p = new Pattern(result[0], 2);
			p.setPatternDetail(result[1]);
			p.setPatternScore(Double.parseDouble(result[2]));
			p.getInfo().put(caseName + "_zscore", Double.parseDouble(result[3]));
			p.getInfo().put(controlName + "_zscore", Double.parseDouble(result[4]));
			
			p.getInfo().put(caseName + "_count", Integer.parseInt(result[5]));
			p.getInfo().put(controlName + "_count", Integer.parseInt(result[6]));
			
			lengthPatterns.add(p);
			
		}
		
		return lengthPatterns;
	}

	public static void rankAndNormalPatterns(ArrayList<Pattern> patterns, boolean isYeast) {
		ArrayList<Pattern> filteredPatterns = new ArrayList<Pattern>();
		
		for (Pattern p : patterns) {
			if (isYeast == false && isBadPattern(p)) {
				System.out.println("Bad Pattern:"+ p);
				continue;
			}
			filteredPatterns.add(p);
		}
		
		patterns.clear();
		patterns.addAll(filteredPatterns);
		
		if (patterns == null || patterns.size() == 0) {
			return;
		}
		
		Collections.sort(patterns, PatternHandler.getPatternScoreComparator());
		Collections.reverse(patterns);
		
		int rank = 1;
		
		double maxPatternScore = patterns.get(0).getPatternScore();
		
		for (Pattern p : patterns) {
			p.setRank(rank++);
			if (isYeast) {
				p.setNormalizedScore(p.getPatternScore() / maxPatternScore);
			} else {
				p.setNormalizedScore((patterns.size() - p.getRank() + 1) * 1.0 / patterns.size());
			}
		}
	}
	
	public static void rankAndNormalRankPatterns(ArrayList<Pattern> patterns) {
		if (patterns == null || patterns.size() == 0) {
			return;
		}
		
		Collections.sort(patterns, PatternHandler.getPatternScoreComparator());
		Collections.reverse(patterns);
		
		int rank = 0;
		
		double maxPatternScore = patterns.get(0).getPatternScore();
		
		for (Pattern p : patterns) {
			p.setNormalizedScore(p.getPatternScore() / maxPatternScore);
			p.setRank(rank++);
		}
		
		System.out.println(rank);
		for (Pattern p : patterns) {
			System.out.println(p.getRank());
			p.setNormalizedScore((rank - p.getRank()) * 1.0 / rank);
			System.out.println(p.getNormalizedScore());
		}
		
	}

	public static Pattern getHighestPattern(ArrayList<Pattern> topPatterns, int[] dt) {
		if (topPatterns == null || dt == null) {
			return null;
		}
		
		for (Pattern d : topPatterns) {
			boolean good = true;
			for (int i = 0; i < d.getSubFeatures().size(); i++) {
				if (dt[d.getSubFeatures().get(i)] != d.getSubIndexs().get(i)) {
					good = false;
					// do not contain the pattern
					break;
				}
			}
			
			if (good == true) {
				// contain the pattern
				return d;
			}
		}
		
		return null;
	}
	
	public static Comparator<Gene> getExpressionComparator(String speciesName, final String cellStatus) {
		if (speciesName.equals("yeast")) {
			return new Comparator<Gene>() {
				@Override
				public int compare(Gene o1, Gene o2) {
					// yeast only have one expression value
					return o1.getExps().get(0).compareTo(o2.getExps().get(0));
				}
			};
		} else {
			return new Comparator<Gene>() {
				@Override
				public int compare(Gene o1, Gene o2) {
					
					if (cellStatus.equals(NucleosomeDataHandler_Human.Cell_Status_Resting)) {
						Double e1 = o1.getExps().get(0) + o1.getExps().get(1);
						Double e2 = o2.getExps().get(0) + o2.getExps().get(1);
						return e1.compareTo(e2);
					} else {
						Double e1 = o1.getExps().get(2) + o1.getExps().get(3);
						Double e2 = o2.getExps().get(2) + o2.getExps().get(3);
						return e1.compareTo(e2);
					}
				}
			};
		}
	}

	public static Comparator<Pattern> getPatternScoreComparator() {
		return new Comparator<Pattern>() {
			@Override
			public int compare(Pattern o1, Pattern o2) {
				return Double.compare(o1.getPatternScore(), o2.getPatternScore());
			}
		};
	}

	public static ArrayList<Pattern> restorePatterns(String caseName, String controlName, String speciesName, String cellStatus) throws IOException {
		String rootFolder = FolderPath.OUTPUT_FOLDER_Full + "pattern_" + speciesName + cellStatus + File.separator;
		String zscoreFolder = rootFolder + "zscore" + File.separator;
		return restoreAllPatternScoreFile(zscoreFolder, caseName, controlName);
	}

	public static HashMap<String, ArrayList<Pattern>> getPatternMap(ArrayList<Pattern> patterns) {
		HashMap<String, ArrayList<Pattern>> patternMap = new HashMap<String, ArrayList<Pattern>>();
		for (Pattern p : patterns) {
			if (patternMap.containsKey(p.toString())) {
				ArrayList<Pattern> ps = patternMap.get(p.toString());
				ps.add(p);
			} else {
				ArrayList<Pattern> ps = new ArrayList<Pattern>();
				ps.add(p);
				patternMap.put(p.toString(), ps);
			}
		}
		return patternMap;
	}

}
