package com.zhengyiyu.handler;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;

import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.obj.Sequence;

public class SequenceReader_Yeast implements SequenceReader{

	// Yuan2005 used this data
	public static String saccer1 = FolderPath.OUTPUT_FOLDER_Full + "/chr_sc_sgd200310/";
	public static String saccer1_subfix = ".fa";

	// Lee2007 used this data
	public static String sgd200602 = FolderPath.OUTPUT_FOLDER_Full + "/chr_sc_sgd200602/";
	public static String sgd200602_subfix = ".fsa";
	
	// the key in the map is the chromosome name, and the value is the whole sequence
	private HashMap<String, String> genomeSequences = new HashMap<String, String>();
	
	/**
	 * @return
	 */
	public static SequenceReader_Yeast getSaccer1Reader() {
		return new SequenceReader_Yeast(saccer1, saccer1_subfix);
	}
	
	/**
	 * @return
	 */
	public static SequenceReader_Yeast getSGD200602Reader() {
		return new SequenceReader_Yeast(sgd200602, sgd200602_subfix);
	}
	
	/**
	 * @param filePosition
	 * @param fileSubfix
	 * for yeast nucleosome, we just read all sequence into the memory
	 */
	private SequenceReader_Yeast(String filePosition, String fileSubfix) {
		super();
		
		for (int i = 1; i <= 16; i++) {
			try {
				BufferedReader reader = new BufferedReader(new FileReader(filePosition + "chr" + i + fileSubfix));
				StringBuffer sb = new StringBuffer();
				
				reader.readLine(); // the first line of the file is an intro line
				String line = "";
				
				while ((line = reader.readLine()) != null) {
					sb.append(line);
				}
				
				reader.close();
				
				genomeSequences.put("chr" + i, sb.toString());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see com.zhengyiyu.handler.SequenceReader#getSequenceBySE(java.lang.String, int, int)
	 */
	public String getSequenceBySE(String chrName, int startPosition, int endPosition) {
		return getSequenceByLength(chrName, startPosition, endPosition - startPosition + 1);
	}
	
	/* (non-Javadoc)
	 * @see com.zhengyiyu.handler.SequenceReader#getSequence(com.zhengyiyu.obj.Sequence)
	 */
	public String getSequence(Sequence seq) {
		return getSequenceByLength(seq.getChromosome(), seq.getStartPosition(), seq.getEndPosition() - seq.getStartPosition() + 1);
	}
	
	/* (non-Javadoc)
	 * @see com.zhengyiyu.handler.SequenceReader#getSequenceByLength(java.lang.String, int, int)
	 */
	public String getSequenceByLength(String chrName, int startPosition, int length) {
		
		if (length < 0) {
			System.out.println("length should not less that 0:" + length);
			return "";
		}
		
		if (startPosition <= 0) {
			startPosition = 1;// the first position of sequence in one chromosome in genome browser is 1;
		}
		
		// the index of string is 0, but the start index of sequence is 1 
		int start = startPosition - 1;
		int end = start + length;
		
		String chrSequence = genomeSequences.get(chrName);
		if (end > chrSequence.length()) {
			// just in case we reach the end of the chromosome
			// as we already stored all the sequence of the yeast genome, it is better to keep them then
			return chrSequence.substring(start, chrSequence.length());
		} else {
			return chrSequence.substring(start, end);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.zhengyiyu.handler.SequenceReader#getChromosomeLength(java.lang.String)
	 */
	public int getChromosomeLength(String chrName) {
		String chrSequence = genomeSequences.get(chrName);
		return chrSequence.length();
	}
}
