package com.zhengyiyu.handler;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import com.zhengyiyu.common.FolderPath;
import com.zhengyiyu.obj.Sequence;
import com.zhengyiyu.util.StringUtil;

/**
 * @author Yiyu Zheng
 *
 */
public class SequenceReader_hg18 implements SequenceReader {
	
	private static int StringLength = 100000; // should be 100000, NTNumPerLine = 50, LineNumPerString = 2000
	
	private String curChromosome;
	private int curChromosomeLength;
	
	private ArrayList<String> sequences = new ArrayList<String>();
	
	private boolean outputChr = true;
	
	private String sequenceFilePath;
	
	private static String DefaultSequenceFilePath = FolderPath.HumanChrFile_Folder_10W;
	
	public static SequenceReader_hg18 humanReader = null;
	
	public static SequenceReader_hg18 getReader() {
		if (humanReader == null) {
			humanReader = new SequenceReader_hg18();
		}
		return humanReader;
	}
	
	public SequenceReader_hg18(boolean outputChr) {
		this(outputChr, DefaultSequenceFilePath);
	}
	
	private SequenceReader_hg18() {
		this(false);
	}
	
	/**
	 * @param outputChr
	 * @param filePath the folder where human sequence are stored
	 */
	private SequenceReader_hg18(boolean outputChr, String filePath) {
		super();
		this.curChromosome = "";
		
		this.outputChr = outputChr;
		this.sequenceFilePath = filePath;
	}

	public String getCurChromesome() {
		return curChromosome;
	}

	public void setCurChromesome(String curChromesome) {
		this.curChromosome = curChromesome;
	}
	
	/* (non-Javadoc)
	 * @see com.zhengyiyu.handler.SequenceReader#getSequenceByLength(java.lang.String, int, int)
	 */
	@Override
	public String getSequenceByLength(String chrName, int startPos, int length) {
		if (length < 0) {
			System.out.println(chrName + ":" + startPos + "-" + (startPos + length - 1));
			System.out.println("Length should not less that 0, current length : " + length);
			return "";
		}
		
		int startPosition = startPos;
		
		if (startPosition <= 0) {
			startPosition = 1;// the first position of sequence in one chromosome in genome browser is 1;
		}
		
		if (!chrName.equals(curChromosome)) {
			loadChromosomeSequence(chrName);
		}
		
		if (startPosition > curChromosomeLength) {
			System.out.println("The position is out of range, current in " + this.curChromosome + ", startPosition: " + startPos);
			return "";
		}
		
		// begin to read
		// the length to be read
		int leftToRead = length;
		
		int endPosition = startPosition + length - 1;
		if (endPosition > curChromosomeLength) {
			System.out.println("The end position is out of range, current in " + this.curChromosome + ", endPosition : " + (startPosition + length));
			leftToRead = curChromosomeLength - startPosition + 1;
		}
		
		startPosition = startPosition - 1; // Tricky, genome starts at 1, while the index of string starts at 0
		
		// tricky the index starts at 0, so 5089 is just in the index = 5 sequence.
		int index = startPosition / StringLength; // 5089 / 1000 = 5
		
		int start = startPosition - index * StringLength; 
		
		int numInCurrentString = sequences.get(index).length() - start;
		
		if (numInCurrentString >= leftToRead) { // changed the > to >=, should be right, do not need to read to the next line if it is just ok
			// it is enough, then...
			return StringUtil.subString(sequences.get(index), start, start + leftToRead);
		} else {
			// System.out.println("need to read two or more lines, " + chrName + ":" + startPosition + "-" + endPosition);

			StringBuffer sb = new StringBuffer();
			sb.append(StringUtil.subString(sequences.get(index++), start, StringLength));
			leftToRead -= numInCurrentString;
			
			int lineToRead = leftToRead / StringLength;
			
			for (int i = 0; i < lineToRead; i++) {
				sb.append(sequences.get(index++));
			}
			// add the left a little
			
			int left = leftToRead % StringLength;

			sb.append(StringUtil.subString(sequences.get(index), 0, left));
			return sb.toString();
		}
	}

	public void loadChromosomeSequence(String chrName) {
		if (outputChr == true) {
			System.out.print("Retrieve Sequence Data for Human " + chrName + ", ");
		}
		curChromosomeLength = 0;
		try {
			sequences.clear();
			sequences = null;
			sequences = new ArrayList<String>();
			
			BufferedReader br = new BufferedReader(new FileReader(this.sequenceFilePath + chrName + ".fa"));
			// do not need to neglect the first line
			String line = "";
			
			while ((line = br.readLine()) != null) {
				if (line.startsWith("###")) {
					continue;
				}
				sequences.add(line);
				curChromosomeLength += line.length();
			}
			
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (outputChr == true) {
			System.out.println("size : " + curChromosomeLength);
		}
		
		curChromosome = chrName;
	}
	
	public String getSequenceBySE(String chrName, int startPosition, int endPosition) {
		return getSequenceByLength(chrName, startPosition, endPosition - startPosition + 1);
	}
	
	public String getSequence(Sequence seq) {
		return getSequenceByLength(seq.getChromosome(), seq.getStartPosition(), seq.length());
	}
	
	public String getSequenceFilePath() {
		return sequenceFilePath;
	}

	public void setSequenceFilePath(String sequenceFilePath) {
		this.sequenceFilePath = sequenceFilePath;
	}

	@Override
	public int getChromosomeLength(String chrName) {
		if (!chrName.equals(curChromosome)) {
			loadChromosomeSequence(chrName);
		}
		return this.curChromosomeLength;
	}
}
