package com.zhengyiyu.handler;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import com.zhengyiyu.feature.SequenceFeature;
import com.zhengyiyu.obj.Pattern;
import com.zhengyiyu.obj.TransitionMatrix;
import com.zhengyiyu.util.ArrayUtil;

/**
 * @author Yiyu
 *
 */
public class ZScoreCalculator {
	public static double calculateZScore(ArrayList<TransitionMatrix> tmes,
			ArrayList<Integer> lengths, ArrayList<Integer> noRepeatLengthList,
			Pattern dimer, int count, String type, double cutoff) throws IOException {
		
		HashMap<Integer, Double> eMap = new HashMap<Integer, Double>();
		// HashMap<Integer, Double> e2Map = new HashMap<Integer, Double>();
		HashMap<Integer, Double> varianceMap = new HashMap<Integer, Double>();
		
		// here I only need to calculate the result for one length then just compare them, 147+146= 293
//		int mostLength = 293;
		
		double e = 0.0;
		double e2 = 0.0;
		double v = 0.0;
		
		double lastE = 0.0;
		boolean notChange = false;
		
		// C3E1S3
		for (int length : noRepeatLengthList) {
			// here we need to handle the length issue
			int l = length - 147 + 1;
			
			if (notChange == true) {
				eMap.put(length, e);
				// e2Map.put(length, e2);
				varianceMap.put(length, v);
			} else {
				e = getExpectation(tmes, dimer, l);
				e2 = getExpectation2(tmes, dimer, l, e);
				
				v = getVariance(e, e2);
				
				eMap.put(length, e);
				// e2Map.put(length, e2);
				varianceMap.put(length, v);
				
			    // System.out.println(length + "\t" + l + "\t" + e + "\t" + e2 + "\t" + v + "\t" + ((e - lastE) / lastE) + "\t" + ((v - lastV) / lastV));
				
				if (l > 147 && (e - lastE) / lastE < cutoff) {
					notChange = true;
				} 
				lastE = e;
			}
		}
		
		ArrayList<Double> eArray = new ArrayList<Double>();
		ArrayList<Double> vArray = new ArrayList<Double>();
		
		for (int length : lengths) {
			eArray.add(eMap.get(length));
			vArray.add(varianceMap.get(length));
		}
		
		double eSum = ArrayUtil.sumDoubleList(eArray);
		double vSum = ArrayUtil.sumDoubleList(vArray);
		
		System.out.print("count:" + count);
		System.out.print(", esum:" + eSum);
		System.out.print(", vSum:" + vSum);
		
		double zscore = calculateScoreBasedOnFormular(count, eSum, vSum);
		System.out.println(", zscore:" + zscore);
		// System.out.println(dimer.toString() + "\t" + zscore + "\t" + eSum + "\t" + e2Sum + "\t" + e + "\t" + e2);
		
		dimer.getInfo().put(type + "_zscore", zscore);
		dimer.getInfo().put(type + "_eSum" + type, eSum);
		dimer.getInfo().put(type + "_vSum" + type, vSum);
		
		return zscore;
	}
	
	public static double calculateZScoreByEstimation(ArrayList<TransitionMatrix> tmes,
			ArrayList<Integer> lengths, ArrayList<Integer> noRepeatLengthList,
			Pattern dimer, int count, String type, double cutoff) throws IOException {
		
		HashMap<Integer, Double> eMap = new HashMap<Integer, Double>();
		// HashMap<Integer, Double> e2Map = new HashMap<Integer, Double>();
		HashMap<Integer, Double> varianceMap = new HashMap<Integer, Double>();
		
		// here I only need to calculate the result for one length then just compare them, 147+146= 293
//		int mostLength = 293;
		
		double e = 0.0;
		double e2 = 0.0;
		double v = 0.0;
		
		double lastE = 0.0;
		boolean notChange = false;
		
		// C3E1S3
		for (int length : noRepeatLengthList) {
			// here we need to handle the length issue
			int l = length - 147 + 1;
			
			if (notChange == true) {
				eMap.put(length, e);
				// e2Map.put(length, e2);
				varianceMap.put(length, v);
			} else {
				e = getExpectation(tmes, dimer, l);
				e2 = getExpectation2(tmes, dimer, l, e);
				
				v = getVariance(e, e2);
				
				eMap.put(length, e);
				// e2Map.put(length, e2);
				varianceMap.put(length, v);
				
//				System.out.println(length + "\t" + l + "\t" + e + "\t" + e2 + "\t" + v + "\t" + ((e - lastE) / lastE) + "\t" + ((v - lastV) / lastV));
				
				if (l > 147 && (e - lastE) / lastE < cutoff) {
					notChange = true;
				} 
				lastE = e;
			}
		}
		
		ArrayList<Double> eArray = new ArrayList<Double>();
		ArrayList<Double> vArray = new ArrayList<Double>();
		
		for (int length : lengths) {
			eArray.add(eMap.get(length));
			vArray.add(varianceMap.get(length));
		}
		
		double eSum = ArrayUtil.sumDoubleList(eArray);
		double vSum = ArrayUtil.sumDoubleList(vArray);
		
		System.out.print("count:" + count);
		System.out.print(", esum:" + eSum);
		System.out.print(", vSum:" + vSum);
		
		double zscore = calculateScoreBasedOnFormular(count, eSum, vSum);
		System.out.println(", zscore:" + zscore);
		// System.out.println(dimer.toString() + "\t" + zscore + "\t" + eSum + "\t" + e2Sum + "\t" + e + "\t" + e2);
		
		dimer.getInfo().put(type + "_zscore", zscore);
		dimer.getInfo().put(type + "_eSum" + type, eSum);
		dimer.getInfo().put(type + "_vSum" + type, vSum);
		
		return zscore;
	}
	
	public static double calculateScoreBasedOnFormular(int count, double eSum, double vSum) {
		return (count - eSum) / Math.sqrt(vSum);
	}
	
	public static double getExpectation(ArrayList<TransitionMatrix> matrixList, Pattern pattern, int hi) {
		//length is the subsequence in the nucleosome, hi = li - 150 + 1;
		
		// step1 seperate pattern
		double result = 0.0;

		// step2 calculation the pi arithmetic product
		double piResult = 1.0;
		for (int i = 0; i < pattern.getSub().size(); i++) {
			int featureIndex = pattern.getSubFeatures().get(i);
			int subIndex = pattern.getSubIndexs().get(i);
			
			double[] pi = matrixList.get(featureIndex).getPi();
			piResult *= pi[subIndex];
		}
		
		result += piResult;
		
		// step3 calculation of the other part
		double hiSumResult = 0.0;
		for (int r = 2; r <= hi; r++) {
			double productResult = 1.0;
			for (int i = 0; i < pattern.getSub().size(); i++) {
				int power = r - 1;
				TransitionMatrix tm = matrixList.get(pattern.getSubFeatures().get(i));
				double sumResult = getSumResult4E(tm, pattern.getSubIndexs().get(i), power); // matrixSize is k
				productResult *= sumResult;
			}
			
			hiSumResult += productResult;
		}
		result+= hiSumResult;
		
		// calculate
		return result;
	}
	
	public static double getExpectationByEstimation(ArrayList<TransitionMatrix> matrixList, Pattern pattern, int hi) {
		//length is the subsequence in the nucleosome, hi = li - 150 + 1;
		
		// step1 seperate pattern
		double result = 0.0;

		// step2 calculation the pi arithmetic product
		double piResult = 1.0;
		for (int i = 0; i < pattern.getSub().size(); i++) {
			int featureIndex = pattern.getSubFeatures().get(i);
			int subIndex = pattern.getSubIndexs().get(i);
			
			double[] pi = matrixList.get(featureIndex).getPi();
			piResult *= pi[subIndex];
		}
		
		result += piResult;
		
		// step3 calculation of the other part
		double hiSumResult = 0.0;
		for (int r = 2; r <= hi; r++) {
			hiSumResult += piResult;
		}
		result+= hiSumResult;
		
		// calculate
		return result;
	}
	
	private static double getSumResult4E(TransitionMatrix tm, int secondStatus, int power) {
		double result = 0.0;
		int k = tm.getPi().length;
		
		double[][] poweredMatrix = tm.getPoweredMatrix(power);
		
		for (int x = 0; x < k; x++) {
			
			// need to do the power of the matrix first
			double pi_t = tm.getPi()[x] * poweredMatrix[x][secondStatus];
			result += pi_t;
		}
		
		return result;
	}
	
	public static double getExpectation2(ArrayList<TransitionMatrix> matrixList, Pattern pattern, int hi, double expection) {
		// length is the subsequence in the nucleosome, hi = li - 150 + 1;
		// step1 seperate pattern
		// step2 calculation of the other part
		
		double rSumResult = 0.0;
		for (int r = 1; r <= hi - 1; r++) {
			double jSumResult = 0.0;
			
			for (int j =  r + 1; j <= hi; j++) {
				
				// third round is add and multiply
				double productResult = 1.0;
				int power1 = r - 1;
				int power2 = j - r;
				
				for (int patternIndex = 0; patternIndex < pattern.getSub().size(); patternIndex++) {
					
					TransitionMatrix tm = matrixList.get(pattern.getSubFeatures().get(patternIndex));
					double sumResult = getSumResult4Expectation2(tm, pattern.getSubIndexs().get(patternIndex), power1, power2); // matrixSize is k
					productResult *= sumResult;
				}
				jSumResult += productResult;
			}
			rSumResult += jSumResult;
		}
		return rSumResult * 2 + expection;
	}
	
	private static double getSumResult4Expectation2(TransitionMatrix tm, Integer secondStatus, int power1, int power2) {
		//length is the subsequence in the nucleosome, hi = li - 150 + 1;
		// step1 seperate pattern
		double result = 0.0;
		
		int k = tm.getPi().length;
		
		// need to find the power matrix first
		double[][] firstPoweredMatrix = tm.getPoweredMatrix(power1);
		
		double[][] secondPoweredMatrix = tm.getPoweredMatrix(power2);
		
		for (int x = 0; x < k; x++) {
			double pi_t = tm.getPi()[x] * firstPoweredMatrix[x][secondStatus] * secondPoweredMatrix[secondStatus][secondStatus];
			result += pi_t;
		}
		
		return result;
	}
	
	private static double getVariance(double e, double e2) {
		return e2 - e * e;
	}
	
	public static void recordZScore(String fileName, ArrayList<Pattern> patterns, ArrayList<SequenceFeature> attrs, String caseName, String controlName) throws IOException {
		
		BufferedWriter bw = new BufferedWriter(new FileWriter(fileName));
		BufferedWriter bwAll = new BufferedWriter(new FileWriter(fileName + "_all"));
		
		bw.write("###Pattern\tDetail\tPatternScore\t" + caseName + "_zscore\t" + controlName + "_zscore\t" + caseName + "_count\t" + controlName + "_count\tindex");
		bw.newLine();
		
		bwAll.write("###Pattern\tDetail\tPatternScore\t" + caseName + "_zscore\t" + controlName + "_zscore\t" + caseName + "_count\t" + controlName + "_count\tindex");
		bwAll.newLine();
		
		int index = 0;
		int allIndex = 0;
		
		for (Pattern d : patterns) {
			int caseCount = (Integer) d.getInfo().get(caseName + "_count");
			int controlCount = (Integer) d.getInfo().get(controlName + "_count");
			
			double caseZScore = (Double) d.getInfo().get(caseName + "_zscore");
			double controlZScore = (Double) d.getInfo().get(controlName + "_zscore");
			
			if (d.getPatternScore() >= PatternHandler.patternScoreThreshold && Double.isInfinite(d.getPatternScore()) == false ) {
				bw.write(d.toString() + "\t" + PatternHandler.translatePattern(attrs, d) + "\t" + d.getPatternScore() + "\t" + + caseZScore + "\t" + controlZScore + "\t" + caseCount + "\t" + controlCount + "\t" + index++);
				bw.newLine();
			}
			
			bwAll.write(d.toString() + "\t" + PatternHandler.translatePattern(attrs, d) + "\t" + d.getPatternScore() + "\t" + + caseZScore + "\t" + controlZScore + "\t" + caseCount + "\t" + controlCount + "\t" + allIndex++);
			bwAll.newLine();
		}
		bw.close();
		bwAll.close();
	}
}
