package com.zhengyiyu.util;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Random;

public class ArrayUtil {
	public static DecimalFormat df2 = new DecimalFormat("0.##");
	public static DecimalFormat df3 = new DecimalFormat("0.###");
	
	public static <T> ArrayList<T> randomPick(ArrayList<T> input, int num) {
		int size = input.size();
		
		if (num >= size) {
			System.out.println("###the size of the original list is less than or equal to the wanting number");
			// So we just return all
			return input;
		}
		
		HashSet<Integer> pickedUpList = new HashSet<Integer>();
		ArrayList<T> output = new ArrayList<T>();
		Random r = new Random();
		
		int index = -1;
		
		for (int i = 0; i < num; i++) {
			do {
				index = r.nextInt(size);
			} while (pickedUpList.contains(index));
			
			output.add(input.get(index));
		}
		return output;
	}
	
	public static <T> ArrayList<T> intervalPick(ArrayList<T> input, int num, int interval) {
		
		ArrayList<T> output = new ArrayList<T>();
		int size = input.size();
		
		if (num * interval > size) {
			System.out.println("the size of the original list is less that wanting number");
			if (num > size) {
				System.out.println("there are only " + size + " items in the input, while we want " + num);
				return input;
			} else {
				interval = size / num;
				System.out.println("\t so the interval is changed to " + interval);
			}
		}
		
		int index = 0;
		
		for (int i = 0; i < num; i++) {
			index += interval;
			output.add(input.get(index));
		}
		return output;
	}
	
	public static void printDoubleList(String name, Iterable<Double> array) {
		System.out.print(name);
		for (double d : array) {
			System.out.print(" " + df2.format(d));
		}
		System.out.println();
	}
	
	public static <E> void printList(String name, Iterable<E> positions) {
		System.out.print(name);
		for (E d : positions) {
			System.out.print("\t" + d);
		}
		System.out.println();
	}
	
	public static <E> void printArray(String name, E[] array) {
		System.out.println(name);
		for (E d : array) {
			System.out.print(" " + d);
			System.out.println();
		}
	}
	
	public static void printArray(String name, double[] array) {
		System.out.println(name);
		for (double d : array) {
			System.out.print(" " + d);
		}
		System.out.println();
	}
	
	public static <E> void printDoubleArray(String name, E[][] array) {
		System.out.println(name);
		for (E[] darray : array) {
			for (E d : darray) {
				System.out.print(" " + d);
			}
			System.out.println();
		}
	}
	
	public static void printDoubleArray(String name, double[][] array) {
		System.out.println(name);
		for (double[] darray : array) {
			for (double d : darray) {
				System.out.print(" " + d);
			}
			System.out.println();
		}
	}
	
	/**
	 * @param array
	 * @return normalize to -1~1
	 */
	public static ArrayList<Double> normalize(ArrayList<Double> array) {
		ArrayList<Double> result = new ArrayList<Double>();
		double max = 0;
		for (double e : array) {
			e = Math.abs(e);
			if (e > max) {
				max = e;
			}
		}
		
		// sub
		if (max != 0) {
			for (Double d : array) {
				result.add(d / max);
			}
		} else {
			return array;
		}
		
		return result;
	}
	
	/**
	 * @param array
	 * @return * @return normalize to -1~1
	 */
	public static ArrayList<Double> normalize2(ArrayList<Double> array) {
		ArrayList<Double> result = new ArrayList<Double>();
		
		double average = ArrayUtil.getAverage(array);
		
		for (int i = 0; i < array.size(); i++) {
			double original = array.get(i);
			array.set(i, original - average);
		}
				
		double max = 0;
		int position = 0;
		int maxPosition = 0;
		
		for (double e : array) {
			e = Math.abs(e);
			if (e > max) {
				max = e;
				maxPosition = position;
			}
			position++;
		}
		
		maxPosition = maxPosition * 1;
		
		// sub
		if (max != 0) {
			for (Double d : array) {
				result.add(d / max);
			}
		} else {
			return array;
		}
		
		return result;
	}
	
	/**
	 * @param array
	 * @return normalize to -1~1
	 */
	public static ArrayList<Double> normalize3(ArrayList<Double> array) {
		ArrayList<Double> result = new ArrayList<Double>();
		
		double average = ArrayUtil.getAverage(array);
		double variance = 0.0;
		for (int i = 0; i < array.size(); i++) {
			double newValue = array.get(i) - average;
			array.set(i, newValue);
			variance = variance + (newValue * newValue);
		}
		
		variance = variance / array.size();

		// to make variance to be 1, we should divide each value by sqrt(variance)
		
		double sqrtVaricance = Math.sqrt(variance);
		
		// sub
		if (sqrtVaricance != 0) {
			for (Double d : array) {
				result.add(d / sqrtVaricance);
			}
		} else {
			return array;
		}
		
		return result;
	}

	public static double getAverage(ArrayList<Double> array) {
		double ave = 0.0;
		
		int size = array.size();
		for (int i = 0; i < size; i++) {
			ave = ave + array.get(i);
		}
		return size > 0 ? ave / size : 0.0;
	}
	
	public static double getIntegerArrayAverage(ArrayList<Integer> array) {
		int sum = 0;
		
		int size = array.size();
		for (int i = 0; i < size; i++) {
			sum = sum + array.get(i);
		}
		return size > 0 ? sum * 1.0 / size : 0;
	}

	public static ArrayList<Double> addDoubleArray(ArrayList<Double> a, ArrayList<Double> b) {
		ArrayList<Double> result = new ArrayList<Double>();
		
		int size = (a.size() > b.size()) ? b.size() : a.size();
		for (int i = 0; i < size; i++) {
			result.add(a.get(i) + b.get(i));
		}
		
		return result;
	}
	
	public static ArrayList<Integer> addIntegerArray(ArrayList<Integer> a, ArrayList<Integer> b) {
		ArrayList<Integer> result = new ArrayList<Integer>();
		
		int size = (a.size() > b.size()) ? b.size() : a.size();
		for (int i = 0; i < size; i++) {
			result.add(a.get(i) + b.get(i));
		}
		
		return result;
	}
	
	public static ArrayList<Double> normalizeIntegerArray(ArrayList<Integer> array) {
		
		ArrayList<Double> result = new ArrayList<Double>();
		
		double average = ArrayUtil.getIntegerArrayAverage(array);
		
		ArrayList<Double> tempArray = new ArrayList<Double>();
		
		for (int i = 0; i < array.size(); i++) {
			int original = array.get(i);
			tempArray.add(original - average);
		}
		
		if (tempArray.size() <= 1) {
			return tempArray;
		}
				
		double max = 0;
		int position = 0;
		int maxPosition = 0;
		
		for (double e : tempArray) {
			e = Math.abs(e);
			if (e > max) {
				max = e;
				maxPosition = position;
			}
			position++;
		}
		
		maxPosition = maxPosition * 1;
		
		// sub
		if (max != 0) {
			for (double d : tempArray) {
				result.add(d / max);
			}
		} else {
			return tempArray;
		}
		
		return result;
	}
	
	public static ArrayList<Double> normalizeIntegerArray2(ArrayList<Integer> array) {
		
		ArrayList<Double> result = new ArrayList<Double>();
		
		double average = ArrayUtil.getIntegerArrayAverage(array);
		
		ArrayList<Double> tempArray = new ArrayList<Double>();
		
		for (int i = 0; i < array.size(); i++) {
			int original = array.get(i);
			tempArray.add(original - average);
		}
		
		if (tempArray.size() <= 2) {
			return tempArray;
		}
		
		// calculate 
		double s2 = 0.0;
		for (double d : tempArray) {
			s2 += d * d;
		}
		
		double s = Math.sqrt(s2 / (tempArray.size() - 1));
			
		for (double d : tempArray) {
			result.add(d / s);
		}
		
		return result;
	}

	public static ArrayList<Double> averageDoubleArray(ArrayList<Double> result, int size) {
		ArrayList<Double> average = new ArrayList<Double>();
		
		for (Double d : result) {
			average.add(d / size);
		}
		return average;
	}
	

	public static String stringSumDoubleList(List<Double> doubleArray) {
		return df2.format(sumDoubleList(doubleArray));
	}
	
	public static double sumDoubleList(List<Double> doubleList) {
		double result = 0.0;
		for (Double d : doubleList) {
			result += d;
		}
		return result;
	}
	
	public static int sumIntegerList(List<Integer> intList) {
		int result = 0;
		for (Integer d : intList) {
			result += d;
		}
		return result;
	}

	public static void normalDistribution(String name, double[] doubleArray) {
		double sum = sumDoubleArray(doubleArray);
		double average = sum / doubleArray.length;
		
		double varience = 0.0;
		for (Double d : doubleArray) {
			varience += (d - average) * (d - average);
		}
		
		varience = varience / doubleArray.length;
		
		System.out.println(name + "\t" + average + "\t" + varience);
		
	}

	private static double sumDoubleArray(double[] doubleArray) {
		double result = 0.0;
		for (Double d : doubleArray) {
			result += d;
		}
		return result;
	}

	public static double[] List2Array(ArrayList<Double> scores) {
		double[] scoreArray = new double[scores.size()];
		for (int i = 0; i < scores.size(); i++) {
			scoreArray[i] = scores.get(i);
		}
		return scoreArray;
	}

	public static ArrayList<Double> Array2List(double[] arrReco) {
		ArrayList<Double> array = new ArrayList<Double>();
		
		for (int i = 0; i < arrReco.length; i++) {
			array.add(arrReco[i]);
		}
		return array;
	}

	public static double getNextMinValue(double[] valueArray, double min) {
		double secondMinValue = Double.MAX_VALUE;
		for (double d : valueArray) {
			if (d > min && d < secondMinValue) {
				secondMinValue = d;
			}
		}
		return secondMinValue;
	}

	public static Double max(ArrayList<Double> localScores) {
		double max = -Double.MAX_VALUE;
		for (Double d : localScores) {
			if (d > max) {
				max = d;
			}
		}
		
		return max;
	}

	public static void normalizeTo1(double[] shape) {
		double sum = ArrayUtil.sumDoubleArray(shape);
		
		if (sum == 0.0) {
			return;
		}
		
		for (int i = 0; i < shape.length; i++) {
			shape[i] = shape[i] / sum;
		}
	}
	
	public static double[] normalizeArray(double[] shape) {
		
		double[] normalizedArray = new double[shape.length];
		
		double sum = ArrayUtil.sumDoubleArray(shape);
		
		if (sum != 0.0) {
			for (int i = 0; i < shape.length; i++) {
				normalizedArray[i] = shape[i] / sum;
			}
		}
		
		return normalizedArray;
	}
	
	public static String array2string(double[] shapeArray, String between) {
		StringBuffer sb = new StringBuffer();
		
		for (double d : shapeArray) {
			sb.append(d + between);
		}
		return sb.toString();
	}

	public static <T> String array2string(T[] shapeArray, String between) {
		StringBuffer sb = new StringBuffer();
		
		for (T d : shapeArray) {
			sb.append(d + between);
		}
		return sb.toString();
	}
	
	public static <T> String list2string(Collection<T> shapeArray, String between) {
		StringBuffer sb = new StringBuffer();
		
		for (T d : shapeArray) {
			sb.append(d + between);
		}
		return sb.toString();
	}
	

}
