/**
 * 
 */
package com.zhengyiyu.util;

import com.zhengyiyu.obj.Sequence;
import com.zhengyiyu.obj.Strand;

/**
 * @author yiyu
 * deal with the relation of two or more regions
 */
public class RegionUtil {

	/**
	 * @param left
	 * @param right
	 * @param start
	 * @param end
	 * @return
	 * 
	 * determine whether [left, right] and [start, end] are overlap
	 */
	public static boolean isOverlap(int left, int right, int start, int end) {
		// 1 bp overlap is overlap! like [10, 20] is overlap with [20, 30]
		
		// left == right means a 1 bp region [20, 20]
		
		// just in case the input parameter is wrong
		if (left > right || start > end) {
			return false;
		}
		
		if (start < left) {
			// [start, end] is in the left of [left, right]
			return end >= left; 
		} else if (start > right) {
			// [start, end] is in the right of [left, right]
			return false;
		} else {
			// overlap
			return true;
		}
	}

	/**
	 * @param seq
	 * @param promoterRegion
	 * @return
	 * 
	 * for a plus strand, it is upstream of the start position, for a minus strand, it is downstream of the end position
	 */
	public static int[] getPromoterRegion(Sequence seq, int[] promoterRegion) {
		return getPromoterRegion(seq.getStrand(), seq.get5PrimeStartPosition(), promoterRegion);
	}
	
	public static int[] getPromoterRegion(Strand strand, int tss, int[] promoterRegion) {
		int startPosition = 0;
		int endPosition = 0;
		
		if (strand == Strand.Plus) {
			if (promoterRegion != null) {
				startPosition = tss + promoterRegion[0];
				if (startPosition <= 0) {
					startPosition = 1;
				}
				endPosition = tss + promoterRegion[1];
			}
		} else if (strand == Strand.Minus) {
			if (promoterRegion != null) {
				startPosition = tss - promoterRegion[1];
				if (startPosition <= 0) {
					startPosition = 1;
				}
				endPosition = tss - promoterRegion[0];
			}
		}
		
		return new int[]{startPosition, endPosition};
	}

}
