package com.zhengyiyu.obj;

import java.util.ArrayList;

public class Gene extends Sequence {
	
	private String geneId;
	private String geneName;
	
	private ArrayList<Double> exps;

	public Gene(String chromosomeName, int startPos, int endPos) {
		super(chromosomeName, startPos, endPos);
		this.exps = new ArrayList<Double>();
	}
	
	public Gene(Sequence seq) {
		super(seq);
		this.exps = new ArrayList<Double>();
	}
	
	public Gene(String geneId, String geneName, Strand strand, String chromosomeName, int startPos, int endPos) {
		super(chromosomeName, startPos, endPos);
		
		this.geneId = geneId;
		this.geneName = geneName;
		this.setStrand(strand);
		this.exps = new ArrayList<Double>();
	}

	@Override
	public String toString() {
		return this.getChromosome() + ":" + this.getStartPosition() + "-" + this.getEndPosition() + "\t" + this.getStrand().toString() + "\t" + this.geneName + "\t" + this.geneId;
	}

	public String getGeneId() {
		return geneId;
	}

	public void setGeneId(String geneId) {
		this.geneId = geneId;
	}

	public String getGeneName() {
		return geneName;
	}

	public void setGeneName(String geneName) {
		this.geneName = geneName;
	}
	
	/**
	 * @return the exps
	 */
	public ArrayList<Double> getExps() {
		return exps;
	}

	/**
	 * @param exps the exps to set
	 */
	public void setExps(ArrayList<Double> exps) {
		this.exps = exps;
	}

	public String toEnsembleFormat() {
		// ENSG00000111780	119368653	119383754	12	GATC
		return this.geneId + "\t" + this.getStartPosition() + "\t" + this.getEndPosition() + "\t" + this.getChromosome().substring(3) + "\t" + this.geneName;
	}
	
	public String toPromoterString(int beginOffset, int endOffset) {
		return "promoter\t" + this.getChromosome() + ":" + (this.getStartPosition() - beginOffset) + "-" + (this.getStartPosition() + endOffset - 1) + "\t" + this.geneName + "\t" + this.geneId;
	}

}
